//______________________________________________________________________
/*                     
                         ∙  ∆ .∴∙≈≡⁞₪⁞≡≈∙∵. ∆  ∙
     ,≈こ≒※⁞⁞░▒▓████▓▒░⁞⁞※∴٭✺ᢩ≡こ意こ≡✺٭∴※⁞⁞⁞⁞░▒▓████▓▒░⁞⁞⁞⁞※≒‗,.        
                  \     _ \   ___|    \     \  |  ___| ____| |     
                 _ \   |   | |       _ \     \ | |     __|   |     
                ___ \  __ <  |      ___ \  |\  | |   | |     |     
              _/    _\_| \_\\____|_/    _\_| \_|\____|_____|_____|          
    `≈≒≡[|⁞⁞░▒▓████▓▒░⁞⁞¦|※‼‖[ ∴✺∵ ]‖‼※|¦⁞⁞░▒▓████▓▒░⁞⁞¦|]≡≒‗,
    ◌ͨ                                                    ᵈᵒᵍᵉᶰᶥᵍᵗ ²ᵒ²₄ 
   ¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯ */
  #include <Arduino.h>
  #include <avr/io.h>
  #include <avr/interrupt.h>
  #include <SoftPWM.h>
  #include <EEPROM.h>
  #include <dht.h>
  #include "TeensyThreads.h"

/*                     _____________
 ✺*∵⁞⁞⁞⁞⁞░░▒▒▓▓███‖[  C O N S T  ]‖███▓▓▒▒░░⁞⁞⁞⁞※≡≒‗,∵ 
                       ¯¯¯¯¯¯¯¯¯¯¯¯¯                       */
  #define VERSION       12.00
  
  #define DEBUG         0
  #define SAFE_MODE     0
  #define WRITE_ROM     1
  
  #define TEMP_LIMIT    40
  #define TEMP_COUNT    60
  #define PIR_TIMEOUT   30
  
  #define coin_key      KEY_C
  #define start_key     KEY_S
  
  #define keyDown       Keyboard.press
  #define keyUp         Keyboard.release
  #define USB           Serial
  #define LCD           Serial1   // TX1 = 1 (CP GREY)
  #define VFD           Serial2   // TX2 = 10 (CP YELLOW#2)
  #define DATE          __DATE__ 
  
  #define f             threads.addThread
  #define wait          threads.delay
  #define fString       __FlashStringHelper*
  #define LengthOf(x)   (sizeof(x)/sizeof(x[0]))
  #define STR(...)      ("" #__VA_ARGS__ )
  #define __FILENAME__ (strrchr("\\" __FILE__, '\\') + 1)
  
  // INPUTS  
  #define DHT_SENS      2         // TEMP/HUMID SENSOR
  #define RESET_BTN     3         // (CP BLACK)    
  #define COIN_INSERT   4         // (COIN BROWN) 5V
  #define ZEROPIN       5         // (COIN RED)
  #define START_BTN     6         // (COIN ORANGE)
  #define FLIP_BEAM     7         // (N/A) 
  #define PIR           8         // (CP GREEN#2)
  #define ENCA          16        // (A2)
  #define ENCB          17        // (A3)
  
  // OUTPUTS
  #define MARQUEE       11        // (CP YELLOW)
  #define COUNTER       12        // (COIN BLUE)
  #define RESET         13
  #define START_BTN_LED 14        // (COIN PURPLE)
  #define COIN_POWER    15        // (COIN GRAY)  
  #define WAKE_LAMP     21        // (CP GREEN)
  #define FAN           22        // (CP YELLOW#2)
  #define USB_POWER     23        // (CP ORANGE)


#if (defined(ARDUINO_TEENSY32) || defined(__MK20DX256__))
  #define ROM_SIZE 2048
#else
  #define ROM_SIZE 512
#endif

//_________________________________________________________________________________

/*                     ___________
 ✺*∵⁞⁞⁞⁞⁞░░▒▒▓▓███‖[  V A R S  ]‖███▓▓▒▒░░⁞⁞⁞⁞※≡≒‗,∵ 
                       ¯¯¯¯¯¯¯¯¯¯¯                      */ 
  enum : int {  Usb, Lcd, Vfd  };

  struct        STRUCT { 
                  uint16_t    num;
                  char const *text; 
                };
                
  byte          inputs = 8;
  int           inPins[] = {
                  RESET_BTN, 
                  COIN_INSERT, 
                  ZEROPIN, 
                  START_BTN, 
                  FLIP_BEAM, 
                  PIR,
                  ENCA,
                  ENCB
                };
                
  STRUCT        inputPins[] = {
                   {DHT_SENS,     "DHT"},
                   {RESET_BTN,    "RESET"},
                   {COIN_INSERT,  "COIN_INSERT"},
                   {ZEROPIN,      "ZEROPIN"},
                   {START_BTN,    "START_BTN"},
                   {FLIP_BEAM,    "FLIP_BEAM"},
                   {PIR,          "PIR"},
                   {ENCA,         "ENCODER A"},
                   {ENCB,         "ENCODER B"}
                };          
  
  byte          outputs = 8;
  int           outPins[] = {
                  MARQUEE, 
                  COUNTER, 
                  RESET, 
                  START_BTN_LED, 
                  COIN_POWER, 
                  WAKE_LAMP, 
                  FAN, 
                  USB_POWER
                };
                
  STRUCT        outputPins[] = {
                   {MARQUEE,        "MARQUEE"},
                   {COUNTER,        "COUNTER"},
                   {RESET,          "RESET"},
                   {START_BTN_LED,  "START_BTN_LED"},
                   {COIN_POWER,     "COIN_POWER"},
                   {WAKE_LAMP,      "WAKE_LAMP"},
                   {FAN,            "FAN"},
                   {USB_POWER,      "USB_POWER"},
                };
  
// ------------------------------------------------------------------------------
  
  // SWITCHES:  // change to bool in the future ?
  byte          debug = DEBUG,
                testMode = DEBUG;
                //writeRom = 0;
  // -----------------------------------------------------
          
  // "BOOLS":   // change to bool in the future      
  byte          first,
                firstBoot,
                asleep,
                booted,
                gameLoaded, 
                sleepSwitch,
                wakeLampFading;
  // -----------------------------------------------------
       
  //LCD:
  #define       BOOTMSGL 15
  byte          lcdReady,
                lcdUpdate,
                bootDataCount,
                bootMsgCount,
                bootMsgPrinted;
                    
  String        bootMsgs[BOOTMSGL] = {},
                LCDsysString0 = "",
                LCDsysString1 = "";
         
  STRUCT        lcd_serial[] = {
                  {0, "SLEEP SWITCH"},
                  {0, "TEMP LIMIT"},
                  {0, "SLEEP TIME"},
                };
  // -----------------------------------------------------
  
  // VFD:
  #define       VFDBOOT 8
  byte          vfdBootCount = 0;
  String        VFDstring = "";   
  // -----------------------------------------------------      
  
  // DHT MODULE
  dht DHT;      // pinMode not needed

  volatile bool temp_first = true;
  float         temp,
                humid;
  
  byte          tempInt, 
                humidInt;
                              
  uint16_t      tempCount = 0,
                tempLimit = TEMP_LIMIT,
                tempMin = 20,
                tempMax = tempLimit,
                humidLimit = 60,
                countLimit = TEMP_COUNT;
  String        tempString;
  // -----------------------------------------------------       
  
  // PIR 
  unsigned int  pir_count   = 0,
                PIR_MSG_LIM = 10000;
  unsigned long pir_timer   = 0,
                pir_msg_cnt = 0;
  byte          pirState,
                pirVal,
                PIR_SENS = 1;
  bool          userAway    = false,
                sys_sleep   = false;
  short         WAKE_UP_KEY = MODIFIERKEY_SHIFT; // KEY_UP
  // -----------------------------------------------------
  
  // Start Btn:
  byte          startValue,
                startLampFading,
                startLampBlinking;
  // -----------------------------------------------------
  
  // Coin:
  unsigned long debounceTime;
  byte const    debounceDelay = 50;
  byte          credits,
                lastCredits,
                update;
  // -----------------------------------------------------
  
  // DC Motor:
  uint16_t      motorTime,
                motorDelay,
                motorSpeed;                               
  byte          motorBusy,
                speedHigh = 255,
                speedLow = 128;
  // -----------------------------------------------------
   
  // Zero Switch:
  #define       ZEROSWITCH 1 
  #define       DEBOUNCE 120
  unsigned long msPrev[ZEROSWITCH];
  byte          zeroPinState,
                zeroTrue,
                zeroState[ZEROSWITCH],
                lastZeroState[ZEROSWITCH];
  // -----------------------------------------------------

  // POWERSAVER:
  enum : int   {OFF, ON};
  bool          MARQUEE_ON = true;
  volatile bool protect;
  volatile int  protect_time = 2000;
  
  // -----------------------------------------------------
  
  // INTERRUPTS:
  // Coin Counter
  #define COIN_DEBOUNCE    100
  volatile unsigned long coinTimer;
  volatile unsigned int  coin_count, last_coin_count;

  // Flip Counter
  #define FLIP_DEBOUNCE    100
  volatile unsigned long flipTimer;
  volatile bool          counting;
  volatile byte          flip_count;

  // Encoder / Spinner
  volatile unsigned long encoderPos;
  unsigned long lastEncoderPos;
  volatile bool spinnerOff = true;
  byte spinnerSens = 32;
  byte spinnerSpeed = 10;
    
  // -----------------------------------------------------
                
  // Fan:
  byte          limitReached = 0,
                fanPwm = 100;
  volatile int  fanPwmFixed = 0;
  
  // -----------------------------------------------------
  
  // Rumble:
  byte          rumbleOn = 1,
                rumbling = 0,
                rumbleRepeats = 1;
  unsigned int  rumbleDuration = 250,
                rumblePreDelay = 250;
  // -----------------------------------------------------
               
  // GAME:
  byte          gameReady,
                gunGame,
                gunBtn,
                initTrigger;            
  String        gameSystem,
                gameName;
  // -----------------------------------------------------
  
  // EEPROM
  String        placeholder;
  uint16_t      eepromOffset;
  byte          dataAddr[5],
                dataCount;
                
  struct        ROMDATA { 
                  uint8_t addr; 
                  String  data; 
                };
                
  ROMDATA       bootdata[4] = {
                  {0, ""},{0, ""},{0, ""},{0, ""},
                };       
  // -----------------------------------------------------

  // CP PANEL BTNS:
  unsigned long btnTimer;
  uint16_t      BTN_TIMEOUT = 1000;
  byte          BTN_MODES   = 3,
                btnMode     = 0;
                
  enum : int  { BTN_LEFT =  1, BTN_MID, BTN_RIGHT };
  enum : int  { MODE_VOLUME, MODE_KEYS, MODE_CREDITS };
  // -----------------------------------------------------
  
  // CP PANEL ANALOG:
  enum : int  { AIN_SWITCH, AIN_POT1, AIN_POT2 };
  
  bool          ainAwait    = false;
  unsigned long ainTimer    = 0;
  int           ain_last    = 3,
                ain_vals[3] = { -1, -1, -1 },
                AIN_TIMEOUT = 10000;
  // -----------------------------------------------------
       
  // VARIOUS
  unsigned long uptime, rstBtnTimer;
  
  char const   *emptyString = "                ";  
  char          datetime[16];  
  
  uint16_t      sleepTime   = PIR_TIMEOUT,
                sleepMin    = 30,
                sleepMax    = 600,
                
                pollCount,
                bootIterations;   

  enum : int   {NONE, TERM, ES, RA, MC};
  int           UI = NONE;

  volatile byte USB_STATE, MARQUEE_STATE;
  byte          COIN_POWER_STATE,
                BYPASS_START,
                PWM_WAKE,
                state_start,
                state_reset,
                state_coin, 
                state_gun;

  // -----------------------------------------------------------------------------------------
/*                _________________________________  
 ✺*∵⁞⁞⁞⁞⁞░▒▓███‖[ F U N C T I O N  D E C L A R E ]‖███▓▒░⁞⁞※≡≒‗,∵ 
                  ¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯¯                */
  void btnPressed(int btn);
  void resetTimers();
  void readLCDInputs();
  void powerSaver(byte state);

  template <class T> String type_name(const T&) { return strtok((strstr((char*)__PRETTY_FUNCTION__, "= ")+2),"]"); }
  
  //__________________________________________________________________________________________
  
  void setup() {    
    USB.begin(9600); // Serial
    LCD.begin(9600); // Serial1 UART
    VFD.begin(9600); // Serial2 UART
    
    SoftPWMBegin();
    
    for (byte o = 0; o<LengthOf(outPins); o++) pinMode(outPins[o], OUTPUT), digitalWrite(outPins[o], LOW);    
    for (byte i = 0; i<LengthOf(inPins); i++)  pinMode(inPins[i], (inPins[i]==7 || inPins[i]==8) ? INPUT : INPUT_PULLUP);
    testOutputs();
    
    f(powerSaver, OFF);
    
    analogWriteFrequency(FAN, 40000); analogWrite(FAN, fanPwm);

    attachInterrupt( digitalPinToInterrupt(COIN_INSERT), coinInserted, FALLING );
    attachInterrupt( digitalPinToInterrupt(FLIP_BEAM),   beamBreak,    FALLING );
    attachInterrupt( digitalPinToInterrupt(ENCA), encoderR, FALLING );
    attachInterrupt( digitalPinToInterrupt(ENCB), encoderL, FALLING );
    delay(1000);
    digitalWrite(RESET, LOW), delay(500), digitalWrite(RESET, HIGH); 

    if (DEBUG) { while (!USB); lcdReady = 1; booted = 1; }
    while (!lcdReady) readLCDSerial();

    checkFirst(); 

    // REM
    /*
    resetTimers(); gameReady = 1;
    digitalWrite(COIN_POWER, COIN_POWER_STATE=1);
    for (;;) {
      if (coin_count != last_coin_count) Serial.println(coin_count), last_coin_count = coin_count;
    }
    */
    // ^REM
        
    printer(Vfd, " ", 1), delay(5000); // NEEDED TO INITIALIZE VFD
    printer(Lcd, "TEENSY READY.", 0);
    printer(Vfd, "A R C A N G E L", 7);
    
    printer(Vfd, "v"+String(VERSION)+" ("+String(datetime).substring(0,8)+")", 1);
    //printer(Vfd, "     v"+String(VERSION), 3); 
    //printer(Vfd, datetime, 1);
    
    resetTimers();
    
    while (!booted) { 
      if ((millis()-uptime) > 20000UL) booted = 1;
      readUsbPort(); 
      readLCDSerial(); 
    }
    digitalWrite(WAKE_LAMP, PWM_WAKE=255); 

    printer(Vfd, " BOOTING SYSTEM", 4);
            
    f(checkPIR);
    f(getTemp);
    f(pollInputs);
    //f(checkAnalog);
    //f(checkCoin);
    //LCD.print("REQ AIN;"); // Request analogInput states from LCD board
  }
  //__________________________________________________________________________________________

  void loop() {
    int count = 0;
    
    gameLoaded = 0;
    gameReady = 0;
    credits = 0;
    state_start = 0;
    state_reset = 0;
    state_coin = 0;
    state_gun = 0;
    gameSystem = "";
    gameName = "";
    
    btnMode = MODE_CREDITS;
    
    digitalWrite(COIN_POWER, COIN_POWER_STATE=0);
    SoftPWMSet(START_BTN_LED, 0);
    
    while (!gameLoaded) { readUsbPort(); readLCDSerial(); }
    
    printer(Vfd, " L O A D I N G", 2);    
    //goToZero();   
    
    while (!gameReady) {
      blinkLamps(START_BTN_LED, 1, 500);
      
      while (count < 5000 && !gameReady){ 
        readUsbPort(), readLCDSerial();
        if (gameReady) break; 
        else count++, delay(1);
      }
      count = 0; 
    }
    
    //f(rumble);
    f(fadeStartBtnLamp);
    
    printer(Vfd, "INSERT COIN!", 6);
    digitalWrite(COIN_POWER, COIN_POWER_STATE=1);   
    goToZero();
    last_coin_count = coin_count;
          
    while (gameLoaded) {
      readUsbPort(), readLCDSerial();
      
      if (gameReady && !protect) {
        if (coin_count != last_coin_count) pressKey(KEY_C), last_coin_count = coin_count; //printer(Usb, coin_count, 1);  // checkingCoin = false, coin_count = credits; checkingCoin = true;
        if (credits != lastCredits)        counter();
      }
    }
    // GAME_ENDED
    digitalWrite(COIN_POWER, COIN_POWER_STATE=0);
    credits = 0; coin_count = 0;

    //if (credits > 0) 
    goToZero();
    testDebug("GAME ENDED.");
  }

  //__________________________________________________________________________________________

  void powerSaver(int state) {
    protect = true;
    wait(1000);

    if (booted) digitalWrite(RESET, LOW);

    if (state == OFF) { 
      if ( (MARQUEE_STATE < 1) && MARQUEE_ON ) digitalWrite(MARQUEE, MARQUEE_STATE=1), wait(protect_time/2);
      if (USB_STATE < 1 ) digitalWrite(USB_POWER, USB_STATE=1);
      fanPwmFixed = -1; temp_first = true;
    }
    else if (state == ON) {
      if (MARQUEE_STATE > 0 ) digitalWrite(MARQUEE, MARQUEE_STATE=0), wait(protect_time/2);
      if (USB_STATE > 0 ) digitalWrite(USB_POWER, USB_STATE=0);
      fanPwmFixed = 0; temp_first = true;
    }
    if (booted) digitalWrite(RESET, HIGH);

    wait(protect_time);
    protect = false;
  }

  //__________________________________________________________________________________________
   
  void coinInserted() {
    if ((millis()-coinTimer) > COIN_DEBOUNCE) {
      if (gameReady && !protect) coin_count++;
      coinTimer = millis();
    }
  }

  //__________________________________________________________________________________________
   
  void beamBreak() {
    if ((millis()-flipTimer) > FLIP_DEBOUNCE) {
      if (counting && !protect) flip_count++;
      flipTimer = millis();
    }
  }

  //__________________________________________________________________________________________

  void keyStrokes(char const *keys) { Keyboard.println(keys); delay(5000); }
  void pressKey(short key) { keyDown(key), delay(250), keyUp(key), delay(250); }

  //__________________________________________________________________________________________

  void encoderL() { if (!spinnerOff && !protect) { for (int x=0; x<spinnerSpeed; x++) Mouse.move(-spinnerSens, 0); } }
  void encoderR() { if (!spinnerOff && !protect) { for (int x=0; x<spinnerSpeed; x++) Mouse.move(spinnerSens, 0); } }

  /*
  void encoderL() { encoderPos--; }
  void encoderR() { encoderPos++; }
  */
  
  //__________________________________________________________________________________________

  void pollInputs() {
    while (1) {
      if (!protect) {
        if (!digitalRead(START_BTN))
          pressKey(KEY_S);
        else if (!digitalRead(RESET_BTN))
          pressKey(KEY_ESC);
      }
    }
  }
  //__________________________________________________________________________________________

  void checkPIR() {
    unsigned int last_count = 0;
    bool sys_sleep = false;
    pir_timer = millis();
     
    while (1) {
      if (booted) {
        if (!pirState && (digitalRead(PIR) == HIGH)) { pirState = 1; 
          while (digitalRead(PIR) == HIGH); 
          if (asleep) pir_count++;
          pir_timer = millis();
        } else { 
          if (pirState) pirState = 0; 
        }
        
        if (asleep) {
          if (pir_count >= PIR_SENS) printer(Usb, "CMD PING", 1), digitalWrite(START_BTN_LED, 1), wait(1000), digitalWrite(START_BTN_LED, 0), pir_count = 0;          
        }

        /*
        T = (unsigned int)((millis()-pir_timer) / 1000);       
        userAway = (bool)(T > PIR_TIMEOUT);
        */
      }
      wait(1);
    }
  }  
     
  //__________________________________________________________________________________________

  void testDebug(String msg) {
    if (DEBUG) printer(Usb, msg, 1);
  }
  //__________________________________________________________________________________________

  void fadeStartBtnLamp() {
    byte i, t = 1; 
    
    SoftPWMSetFadeTime(START_BTN_LED, 500, 1000); SoftPWMSet(START_BTN_LED, 0);
     
    while (gameLoaded) {
      if (credits < 1) {
          for (i=0; i<255; i++){ 
            if (credits > 0) break; 
            SoftPWMSet(START_BTN_LED,(!BYPASS_START)?i:0); wait(t); 
          }
          for (i=255; i>0; i--){ 
            if (credits > 0) break; 
            SoftPWMSet(START_BTN_LED,(!BYPASS_START)?i:0); wait(t);
          }
          if (credits > 0) for (i=i; i<255; i++){ SoftPWMSet(START_BTN_LED,(!BYPASS_START)?i:0), wait(t); }
      } 
      else if (credits > 0) { SoftPWMSet(START_BTN_LED, 255); }
    }
    for (i=i; i>0; i--){ SoftPWMSet(START_BTN_LED,(!BYPASS_START)?i:0), wait(t); } // gameStopped:
    threads.yield();
  }
  //__________________________________________________________________________________________
         
  void fadeWakeLamp() {
    byte t = 1;    

    if (asleep) {
      SoftPWMSetFadeTime(WAKE_LAMP, 500, 1000);

      while (asleep && (PWM_WAKE < 255))   SoftPWMSet(WAKE_LAMP, PWM_WAKE++), wait(t);
      while (asleep) {
        while (asleep && (PWM_WAKE > 0))   SoftPWMSet(WAKE_LAMP, PWM_WAKE--), wait(t);
        while (asleep && (PWM_WAKE < 255)) SoftPWMSet(WAKE_LAMP, PWM_WAKE++), wait(t);
      }
      while (PWM_WAKE < 255) SoftPWMSet(WAKE_LAMP, PWM_WAKE++), wait(t);
    }
  }
  //__________________________________________________________________________________________
  
  void blinkLamps(byte pin, byte times, int del) {
    startLampBlinking = 1;
    digitalWrite(pin, LOW);
    
    for (int i=0; i<times; i++) {
      digitalWrite(pin, HIGH), delay(del);
      digitalWrite(pin, LOW), delay(del);
    }
    startLampBlinking = 0;
  }  
  //__________________________________________________________________________________________

  void readUsbPort() {
    int DELIM, VAL_INT;
    bool echo;
    enum { IN, KEY, VAL, VAR };
    String str[4];
    
    if (USB.available() > 1) {
      str[IN]  = USB.readStringUntil(';');
      DELIM    = str[IN].indexOf(' ');
      str[KEY] = str[IN].substring(0, DELIM);
      str[VAL] = str[IN].substring(DELIM+1, str[IN].length()); 
      str[VAR] = "";
      VAL_INT  = ( isDigit(char(str[VAL][0])) ) ? str[VAL].toInt() : -1;

      if      (str[KEY] == "UI")           UI               = ((VAL_INT > -1)  ?  VAL_INT : UI);
      else if (str[KEY] == "FAN_PWM")      fanPwmFixed      = ((VAL_INT > -1)  ?  VAL_INT : fanPwmFixed),             str[VAR] = String(fanPwmFixed);
      else if (str[KEY] == "SPIN_SENS")    spinnerSens      = ((VAL_INT > 0)   ?  VAL_INT : spinnerSens),             str[VAR] = String(spinnerSens);
      else if (str[KEY] == "SPIN_SPEED")   spinnerSpeed     = ((VAL_INT > 0)   ?  VAL_INT : spinnerSpeed),            str[VAR] = String(spinnerSpeed);
      else if (str[KEY] == "PROTECT_TIME") protect_time     = ((VAL_INT > 0)   ?  VAL_INT : protect_time),            str[VAR] = String(protect_time);
      else if (str[KEY] == "CREDITS")      credits          = ((VAL_INT >= 0)  ?  VAL_INT : credits),                 str[VAR] = String(credits);
      else if (str[KEY] == "TIMEOUT")      sleepTime        = ((VAL_INT >  0)  ?  VAL_INT : sleepTime),               str[VAR] = String(sleepTime); 
      else if (str[KEY] == "PIR_SENS")     PIR_SENS         = ((VAL_INT >  0)  ?  VAL_INT : PIR_SENS),                str[VAR] = String(PIR_SENS); 
      else if (str[KEY] == "GAME_NAME")    gameName         = str[VAL],                                               str[VAR] = String(gameName);
      else if (str[KEY] == "GAME_SYSTEM")  gameSystem       = str[VAL],                                               str[VAR] = String(gameSystem);
      
      else if (str[KEY] == "GAME_READY")   gameReady        = ((VAL_INT > -1)  ?  (bool) VAL_INT : gameReady),        str[VAR] = String(gameReady); 
      else if (str[KEY] == "GAME_LOADED")  gameLoaded       = ((VAL_INT > -1)  ?  (bool) VAL_INT : gameLoaded),       str[VAR] = String(gameLoaded); 
      else if (str[KEY] == "GAME_STOPPED") gameLoaded       = ((VAL_INT > -1)  ?  (bool)!VAL_INT : 0),                str[VAR] = String(!gameLoaded); 
      else if (str[KEY] == "BOOTED")       booted           = ((VAL_INT > -1)  ?  (bool) VAL_INT : booted),           str[VAR] = String(booted); 
      else if (str[KEY] == "ASLEEP")       asleep           = ((VAL_INT > -1)  ?  (bool) VAL_INT : asleep),           str[VAR] = String(asleep);      
      else if (str[KEY] == "USB_POWER")    USB_STATE        = ((VAL_INT > -1)  ?  (bool) VAL_INT : USB_STATE),        str[VAR] = String(USB_STATE);
      else if (str[KEY] == "MARQUEE")      MARQUEE_ON       = ((VAL_INT > -1)  ?  (bool) VAL_INT : MARQUEE_ON),       str[VAR] = String(MARQUEE_ON);
      else if (str[KEY] == "COIN_POWER")   COIN_POWER_STATE = ((VAL_INT > -1)  ?  (bool) VAL_INT : COIN_POWER_STATE), str[VAR] = String(COIN_POWER_STATE);
      else if (str[KEY] == "BYPASS_START") BYPASS_START     = ((VAL_INT > -1)  ?  (bool) VAL_INT : BYPASS_START),     str[VAR] = String(BYPASS_START);
      else if (str[KEY] == "SPIN_OFF")     spinnerOff       = ((VAL_INT > -1)  ?  (bool) VAL_INT : spinnerOff),       str[VAR] = String(spinnerOff);
      
      else if (str[KEY] == "GAME_TITLE")   printer(Vfd, str[VAL], 1); 
      else if (str[KEY] == "VFD")          printer(Vfd, str[VAL].substring(2,str[VAL].length()), (VAL_INT > -1)?VAL_INT:0);
      else if (str[KEY] == "LCD")          printer(Lcd, str[VAL].substring(2,str[VAL].length()), (VAL_INT > -1)?VAL_INT:0);

      // ------------------------------------------------------------------------------
          
      if (str[VAR].length() > 0)                printer(Lcd, str[KEY], 0), printer(Lcd, str[VAR],  1);
      if (str[KEY] != "REQ" )                   printer(Usb, str[IN],  1);
      else {
        if (str[VAL] == "MILLIS")               printer(Usb, "RES MILLIS:"+String(millis()/1000), 1);
        if (str[VAL] == "PIR")                  printer(Usb, "RES PIR:"+String((millis()-pir_timer)/1000), 1);
        if (str[VAL] == "TEMP")                 printer(Usb, "RES TEMP:"+String(tempInt), 1);
      }
 
      if (str[KEY] == "ASLEEP") {
        if (asleep) f(fadeWakeLamp);
        f(powerSaver, (int)asleep);
      }
      if (str[KEY] == "MARQUEE") {
        if (!MARQUEE_ON && (MARQUEE_STATE > 0)) digitalWrite(MARQUEE, MARQUEE_STATE=0);
        else if (MARQUEE_ON && !asleep)         digitalWrite(MARQUEE, MARQUEE_STATE=1);
      }
      if (str[KEY] == "USB_POWER")              digitalWrite(USB_POWER,  USB_STATE);
      if (str[KEY] == "COIN_POWER")             digitalWrite(COIN_POWER, COIN_POWER_STATE);
      if (str[KEY] == "FAN_PWM")                fanPwmFixed = (fanPwmFixed - 1);
    }
}

  //__________________________________________________________________________________________


  void readLCDSerial() {
    String   value, str = "";
    byte index;

    if (LCD.available() > 0) { delay(100);
      if (!lcdReady) lcdReady = 1;

      while (LCD.available() > 0) { 
        char c = LCD.read();
        
        if ((c == '\n') || (c == ';')) break;
        else str += c;
      }
      
      if (str.length() > 1) {
        index = str.indexOf(' ');
         
        if (index > -1) {
          if (str.substring(0, index) == "BTN") {
            if (isDigit( char(str[index+1]) )) {
              str = str.substring(index+1, str.length()).trim();
              btnPressed(str.toInt()); 
            }
          } 
          /*
          else if (str.substring(0, index) == "AIN") {
            if (isDigit( char(str[index+1]) )) {
              str = str.substring(index+1, str.length()).trim();
              index = str.indexOf(' ');
              if (index > -1) {
                value = str.substring(index+1, str.length()).trim();
                str = str.substring(0, index).trim(); // INPUT INDEX
                //ainChanged(str.toInt(), value.toInt());
              }
            }
          }*/
        }
      }
    }
    
  }

  //__________________________________________________________________________________________

  void btnPressed(int btn) {
    if ((millis() - btnTimer) > BTN_TIMEOUT) {
      if ((btn == BTN_MID) && !protect) ++btnMode %= BTN_MODES;
      
      printer(Lcd, "BTN_MODE:", 0);

      switch(btnMode) {
        case MODE_VOLUME: 
          printer(LCD, STR(MODE_VOLUME), 1);
          if ((btn == BTN_RIGHT) && !protect) {
            if (UI == TERM || UI == MC) pressKey(KEY_UP);
            if (UI == ES   || UI == RA) pressKey(KEYPAD_PLUS);
          }
          if ((btn == BTN_LEFT) && !protect) {
            if (UI == TERM || UI == MC) pressKey(KEY_DOWN);
            if (UI == ES   || UI == RA) pressKey(KEYPAD_MINUS);
          }
          break;
        case MODE_CREDITS:
          printer(LCD, STR(MODE_CREDITS), 1);
          if ((UI == RA) && !protect) {
            if (btn == BTN_RIGHT) pressKey(KEY_C); 
            if (btn == BTN_LEFT)  
              if (!BYPASS_START)  pressKey(KEY_S);      
          }
          break;
        case MODE_KEYS:
          printer(LCD, STR(MODE_KEYS), 1);
          if ((btn == BTN_RIGHT) && !protect) {
            if (UI == TERM) keyStrokes("emulationstation");
            if (UI == MC)   pressKey(KEY_F10);
            else            pressKey((UI == RA || gameLoaded) ? KEY_TAB : KEY_ENTER);
          }
          if ((btn == BTN_LEFT) && !protect)  {
            if (UI == TERM) keyStrokes("sudo mc"), UI = MC;
            if (UI == MC)   pressKey(KEY_TAB);
            else            pressKey((UI == RA || gameLoaded) ? KEY_F1 : KEY_SPACE);
          } 
          break;
        default:
          break;
      } 
      if (DEBUG && !protect) Serial.print("btn: "),   Serial.println(btn), Serial.print(", mode: "),Serial.println(btnMode);
      btnTimer = millis();
    }
  }

  //__________________________________________________________________________________________

  void checkAnalog() {
    while (1) {
      if (ainAwait && !protect) {
        if ((millis()-ainTimer) > AIN_TIMEOUT) {
          int sw_val = ain_vals[AIN_SWITCH];
          if      ((sw_val > 1)   && (sw_val <= 100))  sleepSwitch = 1;
          else if ((sw_val > 100) && (sw_val <= 300))  sleepSwitch = 2;
          else if ((sw_val > 300) && (sw_val <= 600))  sleepSwitch = 3;
          else if  (sw_val > 1000)                     sleepSwitch = 0;
          
          printer(Lcd, "SLEEP SWITCH: "+String(sleepSwitch), 0); // printer(1, sleepSwitch, 1); 
          printer(Lcd, "ASLEEP: "+String((asleep)?"TRUE":"FALSE"),1);
          
          ainAwait = false; ain_last = 3;
          if (DEBUG) Serial.print("sleepSwitch: "),Serial.println(sleepSwitch);
          ainTimer = millis();
        }  
      } 
      wait(1);    
    }
  }
  
  void ainChanged(int ain, int val) {
    ain_vals[ain] = val;
    if (ain != ain_last) ainAwait = true, ainTimer = millis();
    ain_last = ain;
  }
  
  //__________________________________________________________________________________________
  
  void goToZero() {
    unsigned long now, motor_start = millis();
    zeroTrue = 0;
    
    while(!zeroTrue || ((millis() - motor_start) > 5000)) { now = millis();
      analogWriteFrequency(COUNTER, 40000); analogWrite(COUNTER, speedHigh);
      
      for (int i=0; i<ZEROSWITCH; i++) { zeroState[i] = digitalRead(ZEROPIN);        
        if ((zeroState[i] == LOW)  && (lastZeroState[i] == HIGH)) msPrev[i] = now;
        if ((zeroState[i] == HIGH) && (lastZeroState[i] == LOW) && (now-msPrev[i]>DEBOUNCE)) zeroTrue = 1;
        lastZeroState[i] = zeroState[i];
      }
    }
    if (!zeroTrue) zeroTrue = 1;
    analogWrite(COUNTER, 0);
  }
  //__________________________________________________________________________________________

  void counter() {
    byte last_flip_count;
    
    if (credits > 0 ) printer(Vfd, "CREDITS: "+String(credits), 4);

    if (credits < lastCredits) 
      goToZero(), delay(500),
      flip_count = 0; 
    else 
      flip_count = lastCredits;
    
    counting = true;
    last_flip_count = flip_count;
    
    while (flip_count < credits) {
      analogWriteFrequency(COUNTER, 1000);
      analogWrite(COUNTER, 128); delay(12); 
      analogWrite(COUNTER, 0);   delay(200); 
    }
    counting = false;
    
    lastCredits = credits;
  }
  //__________________________________________________________________________________________
  
  void printer(byte target, String inputString, byte mode) {
    String outputString;
    unsigned int i = 0;
    
    if (target == 0) {
      outputString = "";
      outputString += inputString;
      if (mode) USB.println(outputString);
      else USB.print(outputString);
      USB.flush();
      
    } else if (target == 1) {
      outputString = "";
      if (mode) outputString += mode;
      else outputString += "0";
      outputString += " ";
      LCD.print(outputString);
      LCD.flush();
      for (i=0; i<inputString.length(); i++) {
        char outChar = inputString[i];
        LCD.write(outChar); LCD.flush();
      }
      LCD.print(";");
      
    } else if (target == 2) {
      outputString = "";
      if (mode) outputString += mode;
      else outputString += "0";
      outputString += " ";
      VFD.print(outputString); VFD.flush();
      for (i=0; i<inputString.length(); i++) {
        char outChar = inputString[i];
        VFD.write(outChar); VFD.flush();
      }
      VFD.print(";");
    }
    outputString = "";
  }
  //__________________________________________________________________________________________
                  
  void getTemp() {
    while (1) { tempCount++;
      if ((tempCount >= countLimit) || temp_first) {
        int dhtData = DHT.read11(DHT_SENS);
        
        temp = ( DHT.temperature - 2.00); tempInt = (temp * 1);
        humid = DHT.humidity; humidInt = (humid * 1);
        tempString = String(tempInt)+char(223)+"C";

        if ((tempInt > tempLimit) || (humid >= humidLimit))
          fanPwm = 255,
          tempLimit = (TEMP_LIMIT * 5);
        else {
          if (fanPwmFixed > -1) fanPwm = fanPwmFixed;
          else fanPwm = map(constrain(tempInt, tempMin, tempMax), tempMin, tempMax, 70, 255);
          tempLimit = TEMP_LIMIT;
        } 
                  
        analogWriteFrequency(FAN, 40000);
        analogWrite(FAN, fanPwm); 
        
        tempCount = 0; temp_first = false;
        limitReached = (int)(tempInt >= tempLimit);
      }
      wait(1000);      
    }
  }
  //__________________________________________________________________________________________
 
  void testOutputs() {
    byte pins[] = {
      START_BTN_LED,
      COIN_POWER,
      WAKE_LAMP,
      FAN
    };
    
    for (byte i=0; i<sizeof pins; i++) digitalWrite(pins[i],HIGH);
    delay(500);
    for (byte i=0; i<sizeof pins; i++) digitalWrite(pins[i],LOW);
  }
  //__________________________________________________________________________________________
    
  String sketchName() {
    String file = F(__FILE__);
    byte pos = 0;
    
    for (byte i=0; i<file.length(); i++)
      if (file[i] == '\\') pos = i;
    pos = pos+1;
    file = file.substring(pos, file.length());
    return file;
  }
  //__________________________________________________________________________________________

  void getDateTime(char *buff) {
    char const *t = __TIME__, *date = __DATE__, month_names[] = "JanFebMarAprMayJunJulAugSepOctNovDec";
    int month, day, year, i;
    
    sscanf(date, "%s %d %d", buff, &day, &year);
    month = (strstr(month_names, buff)-month_names)/3+1;
    sprintf(buff, "%d%02d%02d", year, month, day);
    i = (byte)strlen(buff);
    
    for (byte n=0; n<strlen(t); n++) {
      char c = t[n];
      if (isDigit(c)) buff[i++] = c;
    }
    buff[i++] = '\0';
  }

  //__________________________________________________________________________________________

  void checkFirst() {
    getDateTime(datetime);
    eepromOffset = LengthOf(datetime);
  
    byte b, e = -1, *p = (uint8_t*)datetime - 1;
    
    while ((b = *++p)) {
      if (b != EEPROM[++e]) {
        while ((b = *p++)) EEPROM[e++] = b;
        firstBoot = 1; break;
      }
    }
    /*
    if (firstBoot) {
      printer((DEBUG)?Usb:Lcd, F("timestamp : "), 0); printer((DEBUG)?Usb:Lcd, datetime, 1);
      printer((DEBUG)?Usb:Lcd, F("filename  : "), 0); printer((DEBUG)?Usb:Lcd, __FILENAME__, 1);
      printer((DEBUG)?Usb:Lcd, F("version   : "), 0); printer((DEBUG)?Usb:Lcd, VERSION, 1);
    }
    */
  }

  //__________________________________________________________________________________________

  void eepromHandler() {
    if (firstBoot && WRITE_ROM) writeEEPROM();
    else {
      for (uint8_t i=0; i < sizeof dataAddr; i++) {
        int pos = (2000+i);
        byte dec = EEPROM.read(pos);
        dataAddr[i] = dec;
      }
    }
    for (uint8_t i=0; i < (sizeof dataAddr)-1; i++) {
      byte startPos = dataAddr[i], endPos = dataAddr[i+1], dec;
      char d2c;

      placeholder = ""; 
      
      for (uint16_t j=startPos; j<endPos; j++)
        dec = EEPROM.read(j), d2c = char(dec), placeholder += d2c;
      
      bootdata[i].addr = startPos;
      bootdata[i].data = placeholder;
      if (DEBUG) printer(Usb, "str: ", 0), printer(Usb, placeholder, 1);
    }
    if (firstBoot && WRITE_ROM) dumpEEPROM();
  }
  //__________________________________________________________________________________________
      
  void writeEEPROM() {
    //printer(Vfd, " FLASHING ROM!", 6); 
    if (DEBUG) printer(Usb, "FLASHING ROM", 1);
    dataAddr[dataCount] = 0;
    putData(sketchName());
    putData("v"+String(VERSION));
    putData("Dogenigt");
    dataCount++;
    dataAddr[dataCount] = eepromOffset;
    
    for (uint8_t i=0;i<sizeof dataAddr;i++) {
      byte addr = dataAddr[i];
      int eepromAddr = (2000+i);
      EEPROM.put(eepromAddr, addr);
    }
  }
  //_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _

  void putData(String data) {
    uint16_t dataLen = data.length()+1;
    if (DEBUG) printer(Usb, data, 1);
    EEPROM.put(eepromOffset, data);
    dataCount++;
    dataAddr[dataCount] = eepromOffset;
    eepromOffset += dataLen;
  }
  //__________________________________________________________________________________________
      
  void dumpEEPROM() {
    String testString;

    printer((DEBUG)?Usb:Lcd, "EEPROM: ", 0); delay(1000);
    
    for (byte cnt = 0; cnt < LengthOf(bootdata); cnt++) {
      if (DEBUG) printer(Usb, bootdata[cnt].addr, 0), printer(Usb, " ", 0), printer(Usb, bootdata[cnt].data, 1);
      else printer(Lcd, bootdata[cnt].data, 1), delay(3000), printer(Vfd, bootdata[cnt].data, 1), delay(3000);
    } 
  }
  //__________________________________________________________________________________________
      
 void bootData() {
    byte bootDataMax = 3;
    String dataString = "";
    
    switch (bootDataCount) {
        case 0:
          dataString = "TEMP.: "; dataString += tempString;
          printer(Lcd, dataString, 0);
          dataString = "HUMID: "; dataString += humid; dataString += " %";
          printer(Lcd, dataString, 1);
          break;
        case 1:
          dataString = "MAX TMP: "+String(tempLimit+(char)223)+"C";
          printer(Lcd, dataString, 0);
          dataString = "TIMEOUT: "+String(sleepTime)+"s";
          printer(Lcd, dataString, 1);
          break;
        case 2:
          dataString = "SLEEP MODE: "+String(sleepSwitch);
          printer(Lcd, dataString, 0);
          dataString = "STATE: ";
          if (asleep == 1) dataString += "ASLEEP";
          else dataString += "AWAKE";
          printer(Lcd, dataString, 1);
          break;
        case 3:
          unsigned long teensyTime = (millis() + 1000 )/ 1000;
          
          dataString = "UPTIME: "+String(teensyTime)+"s";
          printer(Lcd, dataString, 0);
          dataString = "tempCount: "+String(tempCount);
          printer(Lcd, dataString, 1);
          break;
      }
      if (bootDataCount >= bootDataMax) bootDataCount = 0;
      else bootDataCount++;
  }
  //__________________________________________________________________________________________

  void resetTimers() {
    uptime      = millis();
    btnTimer    = millis();
    ainTimer    = millis();  
    rstBtnTimer = millis();
    flipTimer   = millis();
    coinTimer   = millis();
  }

  //__________________________________________________________________________________________
  /*
  void screenSaver(byte mode) {
    pollCount++;
    if (pollCount >= 500) {
      if (bootIterations > 5) bootIterations = 0;
      if (mode < 2) VFDscreensaver(bootIterations);
      pollCount = 0;
      bootIterations++;
    }
  }
  //_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
     
  void VFDscreensaver(byte count) {
    byte mode;
    char const *msgs[] = { 
      ". . . . . . . . .",
      "_.,^\\._,-~'),^._/*",
      "_.,^~-.__.,^~-.__.,^~-.__.,^~-._",
      "_.,^\\._,-~),._/*\\_.,(~-._/^._",
      "__.,^*-.__,^*-.__,^*-.__,^*-.__",
      "__,^*-.__        __,^*-.__        __,^*-.__"
    };
    
    switch (count) {
      case 0:  mode=2; break;
      case 1:  mode=2; break;
      case 2:  mode=3; break;
      case 3:  mode=2; break;
      case 4:  mode=2; break;
      default: mode=2; break;
    }
    printer(Vfd, "       "+String(count), 6); delay(1000);
    printer(Vfd, msgs[count],mode); delay(1000);
  }
  //__________________________________________________________________________________________
  */
  //  ___  ____ ___  _  _ ____ ____ ____ ____ 
  //  |  \ |___ |__] |  | | __ | __ |___ |__/ 
  //  |__/ |___ |__] |__| |__] |__] |___ |  \ 
                                            
  void debugger(String arg)  {
    uint16_t del = 2000;
    
    printer(Lcd, "DEBUG: "+String(arg),0); printer(Lcd, "_", 1); delay(1000);

    if (arg == "OUTPUTS") {
      delay(del);
      while (!USB.available()) {
        for (byte cnt=0; cnt < sizeof(outputPins)/sizeof(outputPins[0]); cnt++) {
          byte thisPin = outputPins[cnt].num;
          String thisTxt = outputPins[cnt].text;
          
          if (thisPin != 13) {
            printer(Lcd, String(thisPin)+": "+String(thisTxt),1); delay(del);
            digitalWrite(thisPin, HIGH); delay(del);
            digitalWrite(thisPin, LOW); delay(del); 
          }
        }  
      }
    } else if (arg == "INPUTS") {
      String pinsString = "",
             statesString = "";
           
      for (byte cnt=0; cnt < sizeof(inputPins)/sizeof(inputPins[0]); cnt++) {
        byte thisPin = inputPins[cnt].num;
        
        pinsString += String(thisPin)+" ";
      }  
      printer(Lcd, pinsString,0);
      while (!USB.available()) {
        statesString = "";
        for (byte cnt=0; cnt < sizeof(inputPins)/sizeof(inputPins[0]); cnt++) {
          byte thisPin = inputPins[cnt].num,
               thisState = digitalRead(thisPin);
               
          statesString += String(thisState)+" ";
        }  
        printer(Lcd, statesString,1); delay(1000); 
      }
    } else if (arg == "PINS") {
      Serial.println("INPUTS:");
      for (uint8_t cnt=0; cnt < sizeof(inputPins)/sizeof(inputPins[0]); cnt++) {
        Serial.print(inputPins[cnt].num); Serial.print(": ");
        Serial.println(inputPins[cnt].text);
      }
      Serial.println("\nOUTPUTS:");
      for (uint8_t cnt=0; cnt < sizeof(outputPins)/sizeof(outputPins[0]); cnt++) {
          Serial.print(outputPins[cnt].num);Serial.print(": ");
          Serial.println(outputPins[cnt].text);
      }
    } else if (arg == "VFD") {
      char const *msgs[] = { "ZERO", "ONE", "TWO", "THREE", "FOUR", "FIVE", "SIX", "SEVEN" };
      
      for (byte cnt=0;cnt<=7;cnt++) {
        Serial.print(cnt); Serial.print(" "); Serial.println(msgs[cnt]); Serial.println();
        String vfdMsg = "";
        
        for (byte a=0;a<=2;a++) { vfdMsg += msgs[cnt]; vfdMsg += " "; }
        printer(Vfd, vfdMsg, cnt); delay(5000);
      }
    } else if (arg == "LCD") { 
      while (!USB.available()) {
        readLCDSerial();
        for (byte n = 0; n < LengthOf(lcd_serial); n++) {
          printer(Usb, lcd_serial[n].text, 0); printer(Usb, "\t", 0);printer(Usb, lcd_serial[n].num, 1);
        }
        printer(Usb, "",1); delay(1000);
      }
    } else if (arg == "EEPROM") {
      dumpEEPROM();
    } else if (arg == "TEMP") {
      while (!USB.available()) {
        Serial.print(tempInt),Serial.print("°C ("),Serial.print(humidInt),Serial.println("% humidity)");
        Serial.print("tempLimit: "),Serial.print(tempLimit),Serial.println( (limitReached) ? " (LIMIT REACHED!)" : "");
        Serial.print("FAN PWM:   "),Serial.println(fanPwm);
        Serial.println("");
        delay(1000);
      }
    }
  }

  bool inRange(int base, int drift, int x) {
    int low = (base - drift), high = (base + drift);
    return (x >= low || x <= high);
  }


  //__________________________________________________________________________________________
  
  // ARCADE CONTROL SCRIPT FOR TEENSY v2.0
  // BY D@GENIGT (2021-2024)
  //
