#pragma once

#include <avr/pgmspace.h>
#include <stdint.h>
#include <Arduino.h>

// - - - - - - - - - - - - - - - - - - - - - - 

#define DEBUG       0
#define delayUs     delayMicroseconds
#define randSeed    randomSeed(analogRead(A5))
#define LengthOf(x) (sizeof(x)/sizeof(x[0]))
#define ARGS_N      8
#define CGRAM_N     8
#define BUF_SIZE    64
#define BAUD        9600
#define TIMEOUT     3000

//______________________________________________________________________________

void introText();

//______________________________________________________________________________

enum : bool { ON, OFF };
enum : byte { FADE_IN, FADE_OUT };
enum : byte { PRE_PAD, POST_PAD };
enum : byte { EMPTY, ASCII, CGRAM };
enum : byte { FROM_LEFT, FROM_RIGHT };
enum : byte { DATA_NAN, DATA_PGM, DATA_HEX };
enum : byte {
  ERAS_FADEOUT,
  ERAS_HORI_L, 
  ERAS_HORI_R,
  ERAS_VERT, 
  ERAS_SHIFT_D,
  ERAS_SHIFT_U,
  ERAS_MODES 
};

typedef struct{
  uint8_t  val;
  uint8_t  typ;
} vfd_col;

typedef struct {
  uint8_t glyph[5];
  boolean set;
} vfd_ram;

struct {
  vfd_col col[16];
  vfd_ram ram[8];
  int ram_vacant;
}
vfd_mem;

struct Data {
  char*    cmd;
  char*    msg;
  int      args[ARGS_N+1];
  int      msg_len;
  int      offset;
  byte     args_count;
  byte     clear_mode;
  bool     printing;
  unsigned long timer[2];
}
data;

char buf[BUF_SIZE];
bool incoming;

//______________________________________________________________________________


void readSerial() {
  byte n = 0;
  bool busy = true;
  char c;

  for (int i=0; i<16; i++) 
    vfd_mem.col[i].val = 32,
    vfd_mem.col[i].typ = EMPTY;
 
  memset(buf,0,sizeof(buf)); strcpy(buf,"");
  data.timer[0] = millis();  
  
  while (busy && ((millis()-data.timer[0]) < TIMEOUT)) {
    if (Serial.available() > 0) { c = Serial.read();
      if (c==';' || n >= BUF_SIZE-1)
        busy = false;
      else        
        buf[n++] = c;
    }
  }
  strtok(buf, ";\n\r");
  incoming = false;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

bool checkSerial() {
  bool code = false;
  if (Serial.available()) { 
    char c = Serial.peek();
    if (isalnum(c)) code = true;
    else c = Serial.read();
  }
  return code;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void str2hex( char* str, byte (&bytes)[5] ) {
  char temp[3];
  int i,n,x,len; x = 0;
  len = strlen(str); 

  for (i=0; i<len; i+=2) {
    if ((str[i] == NULL) || (str[i+1] == NULL)) break;

    strncpy(temp, &str[i], 2);

    for (n=0;n<2;n++) 
      temp[n] = (temp[n]>=97 && temp[n]<=122) ? temp[n]-32 : temp[n];

    bytes[x] =  (temp[0]<='9') ? (temp[0]-'0') : (temp[0]-'A'+10); bytes[x] *= 16;
    bytes[x] += (temp[1]<='9') ? (temp[1]-'0') : (temp[1]-'A'+10); x++;
  }
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

uint8_t getVacantRam() {
  uint8_t vacant = 8; // 8 is error code
  for (uint8_t i=0; i<8; i++)
    if (!vfd_mem.ram[i].set)
      return i;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void swap (uint8_t *a, uint8_t *b) {
  uint8_t temp = *a;
  *a = *b;
  *b = temp;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

bool inRange(int low, int high, int x) {
  return  ((x-low) <= (high-low));
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void shiftBit(uint8_t &x, uint8_t bit, bool dir=1) {
  if (dir) x = (x << bit); // left shift
  else     x = (x >> bit); // right shift
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void clearBit(uint8_t &x, uint8_t bit) {
  x &= ~(1 << bit);
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

bool str_match(const char* str, const char* match) {
  return (strcmp(str, match) == 0);
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

char *str_parts(const char *str, bool last=1, const char* delim = " ") {
  return (last) ? ( strchr(str,delim[0]) ? strchr(str,delim[0])+1 : str ) : strtok(str, delim);
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
