#pragma once

#define DIGITS      16    // VFD glyphs
#define DIGITS_H    0x0F  // 15 hexadecimal 

namespace vfd {

  uint8_t brightness = 128;
  
  enum : int {
    DIN   = 11, // DA SDI 
    CLK   = 13, // CK CLK 
    CS    = 10, // CS SPI 
    RST   = 8,  // RS VFD 
    EN    = 7   // EN VFD 
  };
  
  enum : unsigned char {
    ADDR_POS     = 0x20,
    ADDR_ASCII   = 0x30,
    ADDR_CGRAM   = 0x40,
    ADDR_ADRAM   = 0x60,
    ADDR_DIGITS  = 0xE0,
    ADDR_BRIGHT  = 0xE4,
    MODE_NORM    = 0xE8,
    MODE_ALL     = 0xE9, 
    MODE_NONE    = 0xEA,
    MODE_STANDBY = 0xED,
    MODE_RUNNING = 0xEC
  };
  
  void spi_send(uint8_t data) { 
    for (int i=0; i<8; i++, data >>= 1) { // From LSB to MSB
      digitalWrite(DIN, ((data & 0x01) ? 1 : 0));
      digitalWrite(CLK, 0);
      digitalWrite(CLK, 1);
    }
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void serial(bool state) {
    digitalWrite(CS, state);
    delayUs(1);
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void Cmd(byte cmd0) {
    serial(ON);
    spi_send(cmd0); delayUs(1);
    serial(OFF);
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void Cmd(byte cmd0, byte cmd1) {
    serial(ON);
    spi_send(cmd0); delayUs(2);
    spi_send(cmd1); delayUs(1);
    serial(OFF);
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void Show(void) { 
    serial(ON);
    spi_send(MODE_NORM);  
    serial(OFF);
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void Standby(bool on) {
    Cmd((on) ? MODE_STANDBY : MODE_RUNNING);
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void Brightness(byte level) {
    brightness = level;
    Cmd(ADDR_BRIGHT, level);
  }
  // - - - - - - - - - - - - - - - - - - - - - - 

  void Erase(byte pos) {
    Cmd(0x20 | (pos & 0xF), ' '); 
    Show();   
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void Clear() {
    for (int i=0; i<16; i++)
      //Erase(i);
      Cmd(0x20 | (i & 0xF), ' ');
    Show();
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void Write(byte x, byte chr) { 
    serial(ON);
    spi_send(0x20+x);   delayUs(1);
    spi_send(chr+0x30); delayUs(2);
    serial(OFF);
    Show();
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void Print(byte x, char *str) {
    serial(ON);
    spi_send(0x20+x);
  
    while (*str) 
      spi_send(*str),str++;
  
    serial(OFF);
    Show() ;
  }
  // - - - - - - - - - - - - - - - - - - - - - - 

  void Printl(byte pos, char *str, byte len=16) {
    byte n = 0;
    
    serial(ON);
    spi_send(0x20+pos);
    
    while (*str) {
      if (n++ > (len-pos)) break;
      spi_send(*str),
      str++;
    }
    serial(OFF);
    Show() ;
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void SetRam(byte dat, byte flag) {
    serial(ON);
    spi_send(ADDR_ADRAM+dat); //ADRAM
    spi_send(flag);
    serial(OFF);
    Show();
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void Store(byte n, byte *s){
    serial(ON);
    spi_send(0x40+n); delayUs(1);
    for (byte i=0;i<7;i++) 
      delayUs(2), spi_send(s[i]);
    delayUs(1); 
    serial(OFF);
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void Draw(byte p, byte n) {
    serial(ON);
    spi_send(0x20+p); delayUs(2);
    spi_send(0x00+n); delayUs(1);  
    serial(OFF);
    Show();
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  void Init() {
    int pins[5] = { DIN,CLK,CS,RST,EN };
  
    for (int pin : pins) pinMode(pin, OUTPUT);
    
    digitalWrite(EN, 1);
    digitalWrite(RST,0); delayUs(100);
    digitalWrite(RST,1);
  
    Cmd(ADDR_DIGITS, DIGITS_H);
    Brightness(brightness);
    Clear();
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
    
  void Send(byte x=0x00, byte addr=0x00, byte data=0x20) { 
    x &= 0b111111;
      
    if ( x & (1 << 0) ) serial(ON);
    if ( x & (1 << 1) ) spi_send( 0x20 | (addr & 0xF) );
    if ( x & (1 << 2) ) spi_send( data ), delayUs(2);
    if ( x & (1 << 3) ) serial(OFF);
    if ( x & (1 << 4) ) Show();   
  }
  // - - - - - - - - - - - - - - - - - - - - - - 
  
  /*
  void printPad(const char* str, byte n) {
    serial(ON);
    spi_send(0x20);
  
    for (int i = 0; i<n; i++ )
      spi_send(' ');
  
    serial(OFF);
    Show();  
        
    for (; i<16; i++)
      spi_send( (x < strlen(str)) ? str[x++] : ' ' ); 
  }
  */
  void PadLeft(const char* str, byte mode=POST_PAD, byte n=16) {
    int i = 0;
    int x = 0;
    
    serial(ON);
    spi_send( 0x20 );
  
    if (mode == PRE_PAD)
      for (; i<n; i++ )
        spi_send(' ');
  
    for (; i<16; i++)
      spi_send( (x < strlen(str)) ? str[x++] : ' ' );
      
    serial(OFF);
    Show();    
  }
  
  void scrollLeft(char* str, int del=80) {
    for (int i=16; i>=0; i--)
      PadLeft(str, PRE_PAD, i), 
      delay(del);
  
    while (*str) 
      PadLeft(++str),
      delay(del);
  }
  
  void PadRight(const char* str, byte mode=POST_PAD, byte n=16) {
    int i = n;
    int x = strlen(str);
    
    serial(ON);
    spi_send( 0x20 );
  
    if (mode == PRE_PAD)
      for (; i>0; i-- )
        spi_send( (i > x ) ? ' ' : str[x-i] );
  
    if (mode == POST_PAD) {
      for (i=16; i>=0; i--) 
        spi_send( i > x ? ' ' : str[x-i] );
    }
   
    serial(OFF);
    Show();    
  }
  
  void scrollRight(char* str, int del=80) {
    for (int i=0; i<=16; i++)
      PadRight(str, PRE_PAD, i), 
      delay(del);
    
    while (*str) 
      str[ strlen(str)-1 ] = '\0',
      PadRight(str),
      delay(del);
  }

  void Fade(uint8_t dir = FADE_IN, unsigned int spd = 50, uint8_t bright = brightness, bool postClear = false) {
    uint8_t bright_stored = brightness;
    
    if (dir == FADE_IN) {
      for (int count = 0; count <= bright; count++) {
        Brightness(count);
        delay( (1.0*bright/count)*spd/100 );
      }    
    }
    else if (dir == FADE_OUT) {
      for (int count = bright; count > 0; count--) {
        Brightness(count);
        delay( (1.0*bright/count)*spd/100 );
      }
      if (postClear) Clear();
      Brightness(bright_stored);
    }
  }
  
} // <-- end of namespace

//using namespace vfd;
