#include "graphics.h"
#include "font5x7.h"
#include "Globals.h"
#include "VFD16.h"

void introText() {
  const byte order[16] = {1,2,10,0,3,10,4,10,0,5,10,0,6,10,7,10};

  for (byte c=0; c<8; c++) {
    for (byte i=0; i<5; i++)
      vfd_mem.ram[c].glyph[i] = pgm_read_byte((const byte *) &(BIGFONT[c])[i]);
    vfd::Store(c, (const byte*)vfd_mem.ram[c].glyph);
  }
  
  vfd::Clear();

  for (byte n=0; n<16; n++) {
    if (order[n] <8) {
      vfd::Draw(n, order[n]);
    }
  }
  
  delay(3000);
  vfd::Fade(FADE_OUT, 50, vfd::brightness, true);
  delay(1000);
  
  for (int c=0; c<8; c++) 
    vfd::Erase(c);
}

//______________________________________________________________________________

void Store(uint8_t n, PGM_P base) {
  uint8_t *dst = &vfd_mem.ram[n].glyph[0];
  
  for (int i = 0; i < glyphSize; i++) 
    *dst++ = (const uint8_t *)pgm_read_byte( base + i );
  
  vfd::Store(n, (const byte*)vfd_mem.ram[n].glyph);

  vfd_mem.ram[n].set = true;
  vfd_mem.ram_vacant = -1;
  
  for (int i=0; i<CGRAM_N; i++)
    if (vfd_mem.ram[i].set == 0)
      vfd_mem.ram_vacant = i;
  
}
/* ---------------------------------------------------------- */

void setup() {
  Serial.begin(9600);
  
  vfd::Init();
  vfd::Brightness(255);  
  delay(1000);

  introText();  
}
/* ---------------------------------------------------------- */

void storeInput(const char* str) {
  char num[] = "-1";        // Default value for 'cmd' if no space is found
  char *cmd = num;          // Initially point cmd to num
  char *msg = str;          // Assume the whole string is the message initially
  char *delim;
  char *arg = nullptr;      // Initialize to nullptr
  byte val, len;
  int  args_len = ARGS_N+1,
       args[args_len],
       arg_i, n, 
       i = 0;
  
  for (int x=0; x<LengthOf(args); x++) args[x] = -1;
  
  delim = strchr(str, ' '); // Find first space
  
  if (delim) {
    *delim = '\0';          // Replace space with null in 'str'
    cmd = str;              // The first part is the command
    msg = delim + 1;        // The rest is the message
  }
  
  delim = strchr(msg, '$'); // Find first '$'
  
  if (delim) {
    *delim = '\0';          // Replace space with null in 'str'
    arg = delim + 1;        // The part after '$' is args
  } 
  
  if (isDigit(cmd[0])) { arg_i = atoi(cmd);
    int isZero = (strstr(cmd,"0")-cmd)+1; 
    args[0] = (arg_i != 0 || (arg_i == 0 && isZero == 1)) ? arg_i : -1;
  }

  if (arg) {
    char *token = strtok(arg, ","); i=1;
    do {
      if (i<args_len) args[i++] = (isDigit(token[0])) ? atoi(token) : -1;
      token = strtok(nullptr, ",");
    } while (token);
  }

  data.args_count = (i>0) ? i-1 : i;
  
  for (i=0; i<args_len; i++)
    data.args[i] = args[i];

  //data.hasRefs = false;
  len = strlen(msg);
  
  /*
  for (i = 0; i < len; i++) { if (i > 15) break;
    if (msg[i] == 0x25) {
      vfd_mem.col[n].val = ( byte(msg[++i]-48) % 8 );
      vfd_mem.col[n].typ = CGRAM;
      //data.hasRefs = true;
    } else {
      vfd_mem.col[n].val = byte(msg[i]);
      vfd_mem.col[n].typ = ASCII;
    }
    n++;
  }
  */
  
  data.msg_len = len;
  data.offset = 0;
  data.cmd = cmd;
  data.msg = msg;
}
/* ---------------------------------------------------------- */

void WriteCol(uint8_t pos, uint8_t mode=0, uint8_t del=20, uint8_t ram_index = getVacantRam()) {
  uint8_t bytes[5];
  uint8_t glyph[5];
  uint8_t n;
  
  char c = char(vfd_mem.col[pos].val);
  
  if ((vfd_mem.col[pos].typ == ASCII) && (ram_index < 8)) {
    for (int i=0; i<5; i++)
      glyph[i] = pgm_read_byte_near((const byte *) &(font5x7[c])[i]);
  }
  else if (vfd_mem.col[pos].typ == CGRAM) {
    ram_index = vfd_mem.col[pos].val;
    for (int i=0; i<5; i++)
      glyph[i] = vfd_mem.ram[ram_index].glyph[i];
  } 
  else {
    c = char((vfd_mem.col[pos].val >= 32) ? vfd_mem.col[pos].val : 32 );
    vfd::Write(pos, c-48); 
    return;
  }

  if (mode == 2) {
    for (byte i=0; i<5; i++) {
      for (n=0; n<5; n++) 
        bytes[n] = (n <= i) ? glyph[n] : 0x00;
  
      vfd::Store(ram_index,bytes);
      vfd::Draw(pos, ram_index);
      delay(del);
    }
  }
  else if (mode == 3) { 
    for (int i=0; i<5; i++) bytes[i] = glyph[i];
    for (int y=0; y<7; y++) {
      for (int x=0; x<5; x++) {
        bytes[x] = glyph[x];
        for (int z=7; z>y; z--) 
          bytes[x] &= ~(1 << z);
      }
      vfd::Store(ram_index,bytes);
      vfd::Draw(pos, ram_index);
      delay(del);
    }        
  }
  else if (mode == 4) {
    for (n=7; n>0; n--) {
      for (byte i=0; i<5; i++)
        bytes[i] = (glyph[i] >> n);
  
      vfd::Store(ram_index,bytes);
      vfd::Draw(pos, ram_index);
      delay(del);
    }
  }
  vfd::Write(pos, c-48);
}
/* ---------------------------------------------------------- */

void EraseCol(uint8_t pos, uint8_t del=20, uint8_t ram_index = getVacantRam()) {
  uint8_t n, x, cols;
  uint8_t write_count = 0;
  uint8_t bytes[5];
  uint8_t order[5]    = {0,1,2,3,4};
  boolean written[5]  = {0,0,0,0,0};
  
  char c = char(vfd_mem.col[pos].val);
  
  if ((vfd_mem.col[pos].typ == ASCII) && (ram_index < 8)) {
    for (int i=0; i<5; i++)
      bytes[i] = pgm_read_byte_near((const byte *) &(font5x7[c])[i]);
  }
  else if (vfd_mem.col[pos].typ == CGRAM) {
    ram_index = vfd_mem.col[pos].val;
    for (int i=0; i<5; i++)
      bytes[i] = vfd_mem.ram[ram_index].glyph[i];
  } 
  else {
    vfd::Erase(pos); 
    return;
  }

  randSeed;

  for (n=4; n > 0; n--) {
    long r = random(0, n); randSeed;
    swap(&order[n], &order[r]);
  }
  for (int i=0; i<5; i++) 
    if (bytes[i] == 0x00) 
      written[i] = true,
      write_count++;
      
  cols = write_count;
  
  while (write_count < 4) {
    for (int i=0; i<5; i++) {    n = order[i];
      if (written[n] == 0) {     bool found = false;
        while (found == false) { int rn = random(0,8); randSeed;
        
          if ((bytes[n] & (1<<rn)) != 0 ) {
            bytes[n] = (bytes[n] & ~(1 << rn));
            vfd::Store(ram_index,bytes);
            vfd::Draw(pos,ram_index);

            if (bytes[n] == 0x00) {
              written[n] = true;
              write_count++;
            }
            found = true;
          }
        }
        delay(del);
      }
    }
  }
  for (; cols>0; cols--) delay(del);
  vfd::Erase(pos);
}

/* ---------------------------------------------------------- */

void scrollLeft(const char* str, byte mode=POST_PAD, byte n=16) {
  int i = 0;
  int x = 0;
  int len = strlen(str);

  vfd::serial(ON);
  vfd::spi_send( 0x20 );

  if (mode == PRE_PAD)
    for (; i<n; i++ )
      vfd::spi_send(' ');

  for (; i<16; i++) {
    if (str[x] == 0x25) *str++,
      vfd::spi_send( byte(str[x++]-48) );
    else 
      vfd::spi_send((x < len) ? str[x++] : ' ');
  }
          
  vfd::serial(OFF);
  vfd::Show();    
}
/* ---------------------------------------------------------- */

void scrollMain(char* str, int del=80, byte mode=PRE_PAD) {
  if (mode == PRE_PAD) {
    for (int i=16; i>=0; i--)
      scrollLeft(str, PRE_PAD, i), 
      delay(del);
  
    if (strlen > 16) {
      while (*str && (strlen(str)>16))
        scrollLeft(++str),
        delay(del);
    }
  }
  else {
    while (*str)
      scrollLeft(++str),
      delay(del);
  }
}
/* ---------------------------------------------------------- */

void printInput() {
  uint8_t val, i, n;
  uint8_t print_mode = ((data.args[0] > -1) ? data.args[0] : 0);
  uint8_t clear_mode = ((data.args[1] > -1) ? data.args[1] : print_mode);
  uint8_t del_time   = ((data.args[2] > -1) ? data.args[2] : 80);
  uint8_t del_factor = ( del_time / (data.msg_len > 16) ? 16 : data.msg_len );  
  uint8_t brightness = vfd::brightness;

  // WRITE PART
  if (print_mode < 5) {
    if (print_mode == 1) {
      vfd::Brightness(0);
      vfd::Printl(0, data.msg);
      vfd::Fade(FADE_IN, del_time, brightness);
    } 
    else {
      for (i=0; i<data.msg_len; i++) {
        if (i > 16) break;
        WriteCol(i, print_mode, del_factor );
      }      
    }
  }
  else if (print_mode == 5) {
    scrollMain(data.msg, del_time);
  }
  // - - - - - - - - - - - - - - - - - - -
    
  if (clear_mode < 5) {  
    if (strlen(data.msg) > 16) {
      if (clear_mode < 2) 
        delay(2000);

      while (*data.msg && (strlen(data.msg)>16))
        scrollLeft(++data.msg),
        delay(del_time);
    }
    delay(3000);

    if (clear_mode < 2) {
      vfd::Fade(FADE_OUT, del_time, vfd::brightness, true);
    }
    else {
      for (i=0; i<strlen(data.msg); i++) {
        if (i > 16) break;
        EraseCol( i, del_factor );
      }      
    }
  }
  else if (clear_mode == 5) {
    scrollMain(data.msg, del_time, POST_PAD);
  }
}
/* ---------------------------------------------------------- */

bool checkInput() {
  char *split, *val, *key = data.msg;
  char *parts[CGRAM_N];
  
  int val_int, command = data.args[0];
  byte n, ram_n, data_type;
  bool code = 0;
  
  if (command < 0) {
    val = str_parts(data.msg, 1); 
    key = str_parts(data.msg, 0);

    if (str_match(data.cmd, "CMD")) {
      if (str_match(key,"CLS")) vfd::Clear();
    } 
    else
    if (str_match(data.cmd, "PUT")) {
      byte data_type = (
        (str_match(key,"PGM")) ? 1 : 
        (str_match(key,"HEX")) ? 2 : 0
      );

      n = ((data.args[1] > -1) && (data.args[1] < 8)) ? (byte)data.args[1] : 0;
      
      if (data_type > DATA_NAN) {
        ram_n = n; 
        split = strtok(val, " "); 
        do {
          if (n<CGRAM_N) parts[n++] = split; 
          split = strtok(nullptr, " "); 
        } while (split);

        for (; ram_n < n; ram_n++) {
          switch (data_type) {
            case DATA_PGM:
              val_int = ((isDigit(parts[ram_n][0])) ? atoi(parts[ram_n]) : -1);
              if ((val_int > -1) && (val_int < GLYPHS_N)) {
                Store(ram_n, GLYPHS[val_int]);
                vfd_mem.ram[ram_n].set = true;
                vfd::Draw(ram_n, ram_n); // REM
              }
              break;
            case DATA_HEX:
              str2hex(parts[ram_n], vfd_mem.ram[ram_n].glyph);
              vfd_mem.ram[ram_n].set = true;
              break;
            default:
              break;
          } 
        }
      }
    }   
  }
  else if (strlen(data.msg) > 0) {
    code = 1;
  }
  return code;
}
/* ---------------------------------------------------------- */

int setCol(char* str, int n, int i) {
  vfd_col *col = &(vfd_mem.col[n]);

  if (inRange(48, 55, str[i+1]) && (str[i] == 37)) {
    col->val = byte(str[++i]-48);
    col->typ = CGRAM;
  }
  else {
    col->val = (byte)str[i];
    col->typ = ASCII;
  }
  return i;
}
/* ---------------------------------------------------------- */

char* setCols(bool slice=false) {
  char *str = data.msg;
  int len = strlen(str);
  int i, n = 0;
  
  for (i=0; i<len; i++) {
    i = setCol(str, n, i);
    Serial.print(n), Serial.print(": '"), Serial.print(char(vfd_mem.col[n].val)), Serial.print("' "), Serial.print(vfd_mem.col[n].val), Serial.print("\t("), Serial.print( (vfd_mem.col[n].typ == 1) ? "ASCII" : "CGRAM" ), Serial.println(")");
    if ( ++n > 15 ) break;
  }
  for (int x=n; x<16; x++) {
    vfd_mem.col[n].val = 0x20;
    vfd_mem.col[n].typ = EMPTY;
  }
  
  data.offset += i;

  if (slice) {
    for (n=0; n<=i; n++)
      if (*str != '\0') str++;    
  }

  return str+i;
}
/* ---------------------------------------------------------- */

void vfdShift(int n=1) {  
  vfd_col *dst,*src;

  for (int i=0; i<15; i++) {
    dst = &(vfd_mem.col[i]);
    src = &(vfd_mem.col[i+1]);

    *dst = *src;
  }
   vfd_mem.col[15].val = 0x20;
   vfd_mem.col[15].typ = EMPTY;
}
/* ---------------------------------------------------------- */

void loop() {
  byte state = 0;
  int len, i = 0;
  int n = 0;
  incoming = checkSerial();
  
  if (incoming) {
    readSerial();
    storeInput(buf);
    
    if (checkInput()) { 
      len = data.msg_len;
      
      for (i=0; i<len; i++) {
        i = setCol(data.msg, n, i); //Serial.print(n), Serial.print(": '"), Serial.print(char(vfd_mem.col[n].val)), Serial.print("' "), Serial.print(vfd_mem.col[n].val), Serial.print("\t("), Serial.print( (vfd_mem.col[n].typ == 1) ? "ASCII" : "CGRAM" ), Serial.println(")");
        if ( ++n > 16 ) break;
      }
      data.offset = i; //Serial.print("data.msg: '"), Serial.print(data.msg), Serial.print("' ("), Serial.print(data.offset), Serial.println(")");
      vfd::Clear();
      
      for (int y = 0; y<data.offset; y++) {
        if ( y > 16) break;
        WriteCol(y, data.args[0], 50);
      }
      delay(2000); //Serial.print("data.msg: '"), Serial.print(data.msg), Serial.print("' ("), Serial.print(strlen(data.msg)), Serial.println(")");
      
      data.msg_len = strlen(data.msg);
      
      if (data.msg_len > 16) {
        for (n = 16; n < data.msg_len; n++) {
          vfdShift(1); 
          setCol( data.msg, 15, n);
          
          for (int x=0; x<16; x++)
            WriteCol(x, 0, 50);
          delay(100);
        }
      }
      delay(2000);
      
      for (i=0; i<strlen(data.msg); i++) {
        if (i > 16) break;
        EraseCol( i, 10 );
      } 
    }
  }
}

/* ---------------------------------------------------------- */
