#pragma once

/*
   Take 'A' as example.
   'A' use 5 byte to denote:
       0x7C, 0x12, 0x11, 0x12, 0x7C

   and we represent it in base 2:
       0x7C: 01111100
       0x12: 00010010
       0x11: 00010001
       0x12: 00010010
       0x7C: 01111100
   where 1 is font color, and 0 is background color

   So it's 'A' if we look it in counter-clockwise for 90 degree.
   In general case, we also add a background line to seperate from other character:
       0x7C: 01111100
       0x12: 00010010
       0x11: 00010001
       0x12: 00010010
       0x7C: 01111100
       0x00: 00000000

 **/

// standard ascii 5x7 font
// # use font5x7[32] - [127] !
static const PROGMEM byte font5x7[256][5] = {
  { 0x00,  //........ 0x00 (nul)
    0x00,  //........
    0x00,  //........
    0x00,  //........
    0x00   //........
  },
  { 0x3E,  //..#####. 0x01 (soh)
    0x5B,  //.#.##.##
    0x4F,  //.#..####
    0x5B,  //.#.##.##
    0x3E   //..#####.
  },
  { 0x3E,  //..#####. 0x02 (stx)
    0x6B,  //.##.#.##
    0x4F,  //.#..####
    0x6B,  //.##.#.##
    0x3E   //..#####.
  },
  { 0x1C,  //...###.. 0x03 (etx)
    0x3E,  //..#####.
    0x7C,  //.#####..
    0x3E,  //..#####.
    0x1C   //...###..
  },
  { 0x18,  //...##... 0x04 (eot)
    0x3C,  //..####..
    0x7E,  //.######.
    0x3C,  //..####..
    0x18   //...##...
  },
  { 0x1C,  //...###.. 0x05 (enq)
    0x57,  //.#.#.###
    0x7D,  //.#####.#
    0x57,  //.#.#.###
    0x1C   //...###..
  },
  { 0x1C,  //...###.. 0x06 (ack)
    0x5E,  //.#.####.
    0x7F,  //.#######
    0x5E,  //.#.####.
    0x1C   //...###..
  },
  { 0x00,  //........ 0x07 (bel)
    0x18,  //...##...
    0x3C,  //..####..
    0x18,  //...##...
    0x00   //........
  },
  { 0xFF,  //######## 0x08 (bs)
    0xE7,  //###..###
    0xC3,  //##....##
    0xE7,  //###..###
    0xFF   //########
  },
  { 0x00,  //........ 0x09 (tab)
    0x18,  //...##...
    0x24,  //..#..#..
    0x18,  //...##...
    0x00   //........
  },
  { 0xFF,  //######## 0x0A (lf)
    0xE7,  //###..###
    0xDB,  //##.##.##
    0xE7,  //###..###
    0xFF   //########
  },
  { 0x30,  //..##.... 0x0B (vt)
    0x48,  //.#..#...
    0x3A,  //..###.#.
    0x06,  //.....##.
    0x0E   //....###.
  },
  { 0x26,  //..#..##. 0x0C (np)
    0x29,  //..#.#..#
    0x79,  //.####..#
    0x29,  //..#.#..#
    0x26   //..#..##.
  },
  { 0x40,  //.#...... 0x0D (cr)
    0x7F,  //.#######
    0x05,  //.....#.#
    0x05,  //.....#.#
    0x07   //.....###
  },
  { 0x40,  //.#...... 0x0E (so)
    0x7F,  //.#######
    0x05,  //.....#.#
    0x25,  //..#..#.#
    0x3F   //..######
  },
  { 0x5A,  //.#.##.#. 0x0F (si)
    0x3C,  //..####..
    0xE7,  //###..###
    0x3C,  //..####..
    0x5A   //.#.##.#.
  },
  { 0x7F,  //.####### 0x10 (dle)
    0x3E,  //..#####.
    0x1C,  //...###..
    0x1C,  //...###..
    0x08   //....#...
  },
  { 0x08,  //....#... 0x11 (dc1)
    0x1C,  //...###..
    0x1C,  //...###..
    0x3E,  //..#####.
    0x7F   //.#######
  },
  { 0x14,  //...#.#.. 0x12 (dc2)
    0x22,  //..#...#.
    0x7F,  //.#######
    0x22,  //..#...#.
    0x14   //...#.#..
  },
  { 0x5F,  //.#.##### 0x13 (dc3)
    0x5F,  //.#.#####
    0x00,  //........
    0x5F,  //.#.#####
    0x5F   //.#.#####
  },
  { 0x06,  //.....##. 0x14 (dc4)
    0x09,  //....#..#
    0x7F,  //.#######
    0x01,  //.......#
    0x7F   //.#######
  },
  { 0x00,  //........ 0x15 (nak)
    0x66,  //.##..##.
    0x89,  //#...#..#
    0x95,  //#..#.#.#
    0x6A   //.##.#.#.
  },
  { 0x60,  //.##..... 0x16 (syn)
    0x60,  //.##.....
    0x60,  //.##.....
    0x60,  //.##.....
    0x60   //.##.....
  },
  { 0x94,  //#..#.#.. 0x17 (etb)
    0xA2,  //#.#...#.
    0xFF,  //########
    0xA2,  //#.#...#.
    0x94   //#..#.#..
  },
  { 0x08,  //....#... 0x18 (can)
    0x04,  //.....#..
    0x7E,  //.######.
    0x04,  //.....#..
    0x08   //....#...
  },
  { 0x10,  //...#.... 0x19 (em)
    0x20,  //..#.....
    0x7E,  //.######.
    0x20,  //..#.....
    0x10   //...#....
  },
  { 0x08,  //....#... 0x1A (eof)
    0x08,  //....#...
    0x2A,  //..#.#.#.
    0x1C,  //...###..
    0x08   //....#...
  },
  { 0x08,  //....#... 0x1B (esc)
    0x1C,  //...###..
    0x2A,  //..#.#.#.
    0x08,  //....#...
    0x08   //....#...
  },
  { 0x1E,  //...####. 0x1C (fs)
    0x10,  //...#....
    0x10,  //...#....
    0x10,  //...#....
    0x10   //...#....
  },
  { 0x0C,  //....##.. 0x1D (gs)
    0x1E,  //...####.
    0x0C,  //....##..
    0x1E,  //...####.
    0x0C   //....##..
  },
  { 0x30,  //..##.... 0x1E (rs)
    0x38,  //..###...
    0x3E,  //..#####.
    0x38,  //..###...
    0x30   //..##....
  },
  { 0x06,  //.....##. 0x1F (us)
    0x0E,  //....###.
    0x3E,  //..#####.
    0x0E,  //....###.
    0x06   //.....##.
  },
  { 0x00,  //........ 0x20 (space)
    0x00,  //........
    0x00,  //........
    0x00,  //........
    0x00   //........
  },
  { 0x00,  //........ 0x21 !
    0x00,  //........
    0x4F,  //.#..####
    0x00,  //........
    0x00   //........
  },
  { 0x00,  //........ 0x22 "
    0x07,  //.....###
    0x00,  //........
    0x07,  //.....###
    0x00   //........
  },
  { 0x14,  //...#.#.. 0x23 #
    0x7F,  //.#######
    0x14,  //...#.#..
    0x7F,  //.#######
    0x14   //...#.#..
  },
  { 0x24,  //..#..#.. 0x24 $
    0x2A,  //..#.#.#.
    0x7F,  //.#######
    0x2A,  //..#.#.#.
    0x12   //...#..#.
  },
  { 0x23,  //..#...## 0x25 %
    0x13,  //...#..##
    0x08,  //....#...
    0x64,  //.##..#..
    0x62   //.##...#.
  },
  { 0x36,  //..##.##. 0x26 &
    0x49,  //.#..#..#
    0x55,  //.#.#.#.#
    0x22,  //..#...#.
    0x50   //.#.#....
  },
  { 0x00,  //........ 0x27 '
    0x05,  //.....#.#
    0x03,  //......##
    0x00,  //........
    0x00   //........
  },
  { 0x00,  //........ 0x28 (
    0x1C,  //...###..
    0x22,  //..#...#.
    0x41,  //.#.....#
    0x00   //........
  },
  { 0x00,  //........ 0x29 )
    0x41,  //.#.....#
    0x22,  //..#...#.
    0x1C,  //...###..
    0x00   //........
  },
  { 0x14,  //..#...#. 0x2A *
    0x08,  //....#...
    0x3E,  //..#####.
    0x08,  //....#...
    0x14   //..#...#.
  },
  { 0x08,  //....#... 0x2B +
    0x08,  //....#...
    0x3E,  //..#####.
    0x08,  //....#...
    0x08   //....#...
  },
  { 0x00,  //........ 0x2C ,
    0x50,  //#.#.....
    0x30,  //.##.....
    0x00,  //........
    0x00   //........
  },
  { 0x08,  //....#... 0x2D -
    0x08,  //....#...
    0x08,  //....#...
    0x08,  //....#...
    0x08   //....#...
  },
  { 0x00,  //........ 0x2E .
    0x60,  //.##.....
    0x60,  //.##.....
    0x00,  //........
    0x00   //........
  },
  { 0x20,  //..#..... 0x2F /
    0x10,  //...#....
    0x08,  //....#...
    0x04,  //.....#..
    0x02   //......#.
  },
  { 0x3E,  //..#####. 0x30 0
    0x51,  //.#.#...#
    0x49,  //.#..#..#
    0x45,  //.#...#.#
    0x3E   //..#####.
  },
  { 0x00,  //........ 0x31 1
    0x42,  //.#....#.
    0x7F,  //.#######
    0x40,  //.#......
    0x00   //........
  },
  { 0x42,  //.#....#. 0x32 2
    0x61,  //.##....#
    0x51,  //.#.#...#
    0x49,  //.#..#..#
    0x46   //.#...##.
  },
  { 0x21,  //..#....# 0x33 3
    0x41,  //.#.....#
    0x45,  //.#...#.#
    0x4B,  //.#..#.##
    0x31   //..##...#
  },
  { 0x18,  //...##... 0x34 4
    0x14,  //...#.#..
    0x12,  //...#..#.
    0x7F,  //.#######
    0x10   //...#....
  },
  { 0x27,  //..#..### 0x35 5
    0x45,  //.#...#.#
    0x45,  //.#...#.#
    0x45,  //.#...#.#
    0x39   //..###..#
  },
  { 0x3C,  //..####.. 0x36 6
    0x4A,  //.#..#.#.
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x30   //..##....
  },
  { 0x01,  //.......# 0x37 7
    0x71,  //.###...#
    0x09,  //....#..#
    0x05,  //.....#.#
    0x03   //......##
  },
  { 0x36,  //..##.##. 0x38 8
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x36   //..##.##.
  },
  { 0x06,  //.....##. 0x39 9
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x29,  //..#.#..#
    0x1E   //...####.
  },
  { 0x00,  //........ 0x3A :
    0x36,  //.##.##..
    0x36,  //.##.##..
    0x00,  //........
    0x00   //........
  },
  { 0x00,  //........ 0x3B ;
    0x56,  //.#.#.##.
    0x36,  //..##.##.
    0x00,  //........
    0x00   //........
  },
  { 0x08,  //....#... 0x3C <
    0x14,  //...#.#..
    0x22,  //..#...#.
    0x41,  //.#.....#
    0x00   //........
  },
  { 0x14,  //...#.#.. 0x3D =
    0x14,  //...#.#..
    0x14,  //...#.#..
    0x14,  //...#.#..
    0x14   //...#.#..
  },
  { 0x00,  //........ 0x3E >
    0x41,  //.#.....#
    0x22,  //..#...#.
    0x14,  //...#.#..
    0x08   //....#...
  },
  { 0x02,  //......#. 0x3F ?
    0x01,  //.......#
    0x51,  //.#.#...#
    0x09,  //....#..#
    0x06   //.....##.
  },
  { 0x32,  //..##..#. 0x40 @
    0x49,  //.#..#..#
    0x79,  //.####..#
    0x41,  //.#.....#
    0x3E   //..#####.
  },
  { 0x7E,  //.#####.. 0x41 A
    0x11,  //...#...#
    0x11,  //...#...#
    0x11,  //...#...#
    0x7E   //.#####..
  },
  { 0x7F,  //.####### 0x42 B
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x36   //..##.##.
  },
  { 0x3E,  //..#####. 0x43 C
    0x41,  //.#.....#
    0x41,  //.#.....#
    0x41,  //.#.....#
    0x22   //..#...#.
  },
  { 0x41,  //.#.....# 0x44 D
    0x7F,  //.#######
    0x41,  //.#.....#
    0x41,  //.#.....#
    0x3E   //..#####.
  },
  { 0x7F,  //.####### 0x45 E
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x41   //.#.....#
  },
  { 0x7F,  //.####### 0x46 F
    0x09,  //....#..#
    0x09,  //....#..#
    0x09,  //....#..#
    0x01   //.......#
  },
  { 0x3E,  //..#####. 0x47 G
    0x41,  //.#.....#
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x78   //.####...
  },
  { 0x7F,  //.####### 0x48 H
    0x08,  //....#...
    0x08,  //....#...
    0x08,  //....#...
    0x7F   //.#######
  },
  { 0x00,  //........ 0x49 I
    0x41,  //.#.....#
    0x7F,  //.#######
    0x41,  //.#.....#
    0x00   //........
  },
  { 0x20,  //..#..... 0x4A J
    0x40,  //.#......
    0x41,  //.#.....#
    0x3F,  //..######
    0x01   //.......#
  },
  { 0x7F,  //.####### 0x4B K
    0x08,  //....#...
    0x14,  //...#.#..
    0x22,  //..#...#.
    0x41   //.#.....#
  },
  { 0x7F,  //.####### 0x4C L
    0x40,  //.#......
    0x40,  //.#......
    0x40,  //.#......
    0x40   //.#......
  },
  { 0x7F,  //.####### 0x4D M
    0x02,  //......#.
    0x0C,  //...###..
    0x02,  //......#.
    0x7F   //.#######
  },
  { 0x7F,  //.####### 0x4E N
    0x04,  //.....#..
    0x08,  //....#...
    0x10,  //...#....
    0x7F   //.#######
  },
  { 0x3E,  //..#####. 0x4F O
    0x41,  //.#.....#
    0x41,  //.#.....#
    0x41,  //.#.....#
    0x3E   //..#####.
  },
  { 0x7F,  //.####### 0x50 P
    0x09,  //....#..#
    0x09,  //....#..#
    0x09,  //....#..#
    0x06   //.....##.
  },
  { 0x3E,  //..#####. 0x51 Q
    0x41,  //.#.....#
    0x51,  //.#.#...#
    0x21,  //..#....#
    0x5E   //.#.####.
  },
  { 0x7F,  //.####### 0x52 R
    0x09,  //....#..#
    0x19,  //...##..#
    0x29,  //..#.#..#
    0x46   //.#...##.
  },
  { 0x46,  //.#...##. 0x53 S
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x31   //..##...#
  },
  { 0x01,  //.......# 0x54 T
    0x01,  //.......#
    0x7F,  //.#######
    0x01,  //.......#
    0x01   //.......#
  },
  { 0x3F,  //..###### 0x55 U
    0x40,  //.#......
    0x40,  //.#......
    0x40,  //.#......
    0x3F   //..######
  },
  { 0x1F,  //...##### 0x56 V
    0x20,  //..#.....
    0x40,  //.#......
    0x20,  //..#.....
    0x1F   //...#####
  },
  { 0x3F,  //..###### 0x57 W
    0x40,  //.#......
    0x38,  //..###...
    0x40,  //.#......
    0x3F   //..######
  },
  { 0x63,  //.##...## 0x58 X
    0x14,  //...#.#..
    0x08,  //....#...
    0x14,  //...#.#..
    0x63   //.##...##
  },
  { 0x07,  //.....### 0x59 Y
    0x08,  //....#...
    0x70,  //.####...
    0x08,  //....#...
    0x07   //.....###
  },
  { 0x61,  //.##....# 0x5A Z
    0x51,  //.#.##..#
    0x49,  //.#..#..#
    0x45,  //.#..##.#
    0x43   //.#....##
  },
  { 0x00,  //........ 0x5B [
    0x7F,  //.#######
    0x41,  //.#.....#
    0x41,  //.#.....#
    0x00   //........
  },
  { 0x02,  //......#. 0x5C backslash
    0x04,  //.....#..
    0x08,  //....#...
    0x10,  //...#....
    0x20   //..#.....
  },
  { 0x00,  //........ 0x5D ]
    0x41,  //.#.....#
    0x41,  //.#.....#
    0x7F,  //.#######
    0x00   //........
  },
  { 0x04,  //.....#.. 0x5E ^
    0x02,  //......#.
    0x01,  //.......#
    0x02,  //......#.
    0x04   //.....#..
  },
  { 0x40,  //.#...... 0x5F _
    0x40,  //.#......
    0x40,  //.#......
    0x40,  //.#......
    0x40   //.#......
  },
  { 0x00,  //........ 0x60 `
    0x01,  //.......#
    0x02,  //......#.
    0x04,  //.....#..
    0x00   //........
  },
  { 0x20,  //..#..... 0x61 a
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x78   //.####...
  },
  { 0x7F,  //.####### 0x62 b
    0x48,  //..#.#...
    0x44,  //.#...#..
    0x44,  //.#...#..
    0x38   //..###...
  },
  { 0x38,  //..###... 0x63 c
    0x44,  //.#...#..
    0x44,  //.#...#..
    0x44,  //.#...#..
    0x20   //..#.....
  },
  { 0x38,  //..###... 0x64 d
    0x44,  //.#...#..
    0x44,  //.#...#..
    0x48,  //.#..#...
    0x7F   //.#######
  },
  { 0x38,  //..###... 0x65 e
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x18   //...##...
  },
  { 0x08,  //....#... 0x66 f
    0x7E,  //.######.
    0x09,  //....#..#
    0x01,  //.......#
    0x02   //......#.
  },
  { 0x0C,  //....##.. 0x67 g
    0x52,  //.#.#..#.
    0x52,  //.#.#..#.
    0x52,  //.#.#..#.
    0x3E   //..#####.
  },
  { 0x7F,  //.####### 0x68 h
    0x08,  //....#...
    0x04,  //.....#..
    0x04,  //.....#..
    0x78   //.####...
  },
  { 0x00,  //........ 0x69 i
    0x44,  //.#...#..
    0x7D,  //.#####.#
    0x40,  //.#......
    0x00   //........
  },
  { 0x20,  //..#..... 0x6A j
    0x40,  //.#......
    0x44,  //.#...#..
    0x3D,  //..####.#
    0x00   //........
  },
  { 0x7F,  //.####### 0x6B k
    0x10,  //...#....
    0x28,  //..#.#...
    0x44,  //.#...#..
    0x00   //........
  },
  { 0x00,  //........ 0x6C l
    0x41,  //.#.....#
    0x7F,  //.#######
    0x40,  //.#......
    0x00   //........
  },
  { 0x7C,  //.#####.. 0x6D m
    0x04,  //.....#..
    0x18,  //...##...
    0x04,  //.....#..
    0x78   //.####...
  },
  { 0x7C,  //.#####.. 0x6E n
    0x08,  //....#...
    0x04,  //.....#..
    0x04,  //.....#..
    0x78   //.####...
  },
  { 0x38,  //..###... 0x6F o
    0x44,  //.#...#..
    0x44,  //.#...#..
    0x44,  //.#...#..
    0x38   //..###...
  },
  { 0x7C,  //.#####.. 0x70 p
    0x14,  //...#.#..
    0x14,  //...#.#..
    0x14,  //...#.#..
    0x08   //....#...
  },
  { 0x08,  //....#... 0x71 q
    0x14,  //...#.#..
    0x14,  //...#.#..
    0x18,  //...##...
    0x7C   //.#####..
  },
  { 0x7C,  //.#####.. 0x72 r
    0x08,  //....#...
    0x04,  //.....#..
    0x04,  //.....#..
    0x08   //....#...
  },
  { 0x48,  //.#..#... 0x73 s
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x20   //..#.....
  },
  { 0x04,  //.....#.. 0x74 t
    0x3F,  //..######
    0x44,  //.#...#..
    0x40,  //.#......
    0x20   //..#.....
  },
  { 0x3C,  //..####.. 0x75 u
    0x40,  //.#......
    0x40,  //.#......
    0x20,  //..#.....
    0x7C   //.#####..
  },
  { 0x1C,  //...###.. 0x76 v
    0x20,  //..#.....
    0x40,  //.#......
    0x20,  //..#.....
    0x1C   //...###..
  },
  { 0x3C,  //..####.. 0x77 w
    0x40,  //.#......
    0x30,  //..##....
    0x40,  //.#......
    0x3C   //..####..
  },
  { 0x44,  //.#...#.. 0x78 x
    0x28,  //..#.#...
    0x10,  //...#....
    0x28,  //..#.#...
    0x44   //.#...#..
  },
  { 0x0C,  //....##.. 0x79 y
    0x50,  //.#.#....
    0x50,  //.#.#....
    0x50,  //.#.#....
    0x3C   //..####..
  },
  { 0x44,  //.#...#.. 0x7A z
    0x64,  //.##..#..
    0x54,  //.#.#.#..
    0x4C,  //.#..##..
    0x44   //.#...#..
  },
  { 0x00,  //........ 0x7B {
    0x08,  //....#...
    0x36,  //..##.##.
    0x41,  //.#.....#
    0x00   //........
  },
  { 0x00,  //........ 0x7C |
    0x00,  //........
    0x7F,  //.#######
    0x00,  //........
    0x00   //........
  },
  { 0x00,  //........ 0x7D }
    0x41,  //.#.....#
    0x36,  //..##.##.
    0x08,  //....#...
    0x00   //........
  },
  { 0x02,  //......#. 0x7E ~
    0x01,  //.......#
    0x03,  //......##
    0x02,  //......#.
    0x01   //.......#
  },
  { 0x3C,  //..####.. 0x7F
    0x26,  //..#..##.
    0x23,  //..#...##
    0x26,  //..#..##.
    0x3C   //..####..
  },
  { 0x1E,  //...####. 0x80
    0xA1,  //#.#....#
    0xA1,  //#.#....#
    0x61,  //.##....#
    0x12   //...#..#.
  },
  { 0x3A,  //..###.#. 0x81
    0x40,  //.#......
    0x40,  //.#......
    0x20,  //..#.....
    0x7A   //.####.#.
  },
  { 0x38,  //..###... 0x82
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x55,  //.#.#.#.#
    0x59   //.#.##..#
  },
  { 0x21,  //..#....# 0x83
    0x55,  //.#.#.#.#
    0x55,  //.#.#.#.#
    0x79,  //.####..#
    0x41   //.#.....#
  },
  { 0x22,  //..#...#. 0x84
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x78,  //.####...
    0x42   //.#....#.
  },
  { 0x21,  //..#....# 0x85
    0x55,  //.#.#.#.#
    0x54,  //.#.#.#..
    0x78,  //.####...
    0x40   //.#......
  },
  { 0x20,  //..#..... 0x86
    0x54,  //.#.#.#..
    0x55,  //.#.#.#.#
    0x79,  //.####..#
    0x40   //.#......
  },
  { 0x0C,  //....##.. 0x87
    0x1E,  //...####.
    0x52,  //.#.#..#.
    0x72,  //.###..#.
    0x12   //...#..#.
  },
  { 0x39,  //..###..# 0x88
    0x55,  //.#.#.#.#
    0x55,  //.#.#.#.#
    0x55,  //.#.#.#.#
    0x59   //.#.##..#
  },
  { 0x39,  //..###..# 0x89
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x59   //.#.##..#
  },
  { 0x39,  //..###..# 0x8A
    0x55,  //.#.#.#.#
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x58   //.#.##...
  },
  { 0x00,  //........ 0x8B
    0x00,  //........
    0x45,  //.#...#.#
    0x7C,  //.#####..
    0x41   //.#.....#
  },
  { 0x00,  //........ 0x8C
    0x02,  //......#.
    0x45,  //.#...#.#
    0x7D,  //.#####.#
    0x42   //.#....#.
  },
  { 0x00,  //........ 0x8D
    0x01,  //.......#
    0x45,  //.#...#.#
    0x7C,  //.#####..
    0x40   //.#......
  },
  { 0x7D,  //.#####.# 0x8E
    0x12,  //...#..#.
    0x11,  //...#...#
    0x12,  //...#..#.
    0x7D   //.#####.#
  },
  { 0xF0,  //####.... 0x8F
    0x28,  //..#.#...
    0x25,  //..#..#.#
    0x28,  //..#.#...
    0xF0   //####....
  },
  { 0x7C,  //.#####.. 0x90
    0x54,  //.#.#.#..
    0x55,  //.#.#.#.#
    0x45,  //.#...#.#
    0x00   //........
  },
  { 0x20,  //..#..... 0x91
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x7C,  //.#####..
    0x54   //.#.#.#..
  },
  { 0x7C,  //.#####.. 0x92
    0x0A,  //....#.#.
    0x09,  //....#..#
    0x7F,  //.#######
    0x49   //.#..#..#
  },
  { 0x32,  //..##..#. 0x93
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x32   //..##..#.
  },
  { 0x3A,  //..###.#. 0x94
    0x44,  //.#...#..
    0x44,  //.#...#..
    0x44,  //.#...#..
    0x3A   //..###.#.
  },
  { 0x32,  //..##..#. 0x95
    0x4A,  //.#..#.#.
    0x48,  //.#..#...
    0x48,  //.#..#...
    0x30   //..##....
  },
  { 0x3A,  //..###.#. 0x96
    0x41,  //.#.....#
    0x41,  //.#.....#
    0x21,  //..#....#
    0x7A   //.####.#.
  },
  { 0x3A,  //..###.#. 0x97
    0x42,  //.#....#.
    0x40,  //.#......
    0x20,  //..#.....
    0x78   //.####...
  },
  { 0x00,  //........ 0x98
    0x9D,  //#..###.#
    0xA0,  //#.#.....
    0xA0,  //#.#.....
    0x7D   //.#####.#
  },
  { 0x3D,  //..####.# 0x99
    0x42,  //.#....#.
    0x42,  //.#....#.
    0x42,  //.#....#.
    0x3D   //..####.#
  },
  { 0x3D,  //..####.# 0x9A
    0x40,  //.#......
    0x40,  //.#......
    0x40,  //.#......
    0x3D   //..####.#
  },
  { 0x3C,  //..####.. 0x9B
    0x24,  //..#..#..
    0xFF,  //########
    0x24,  //..#..#..
    0x24   //..#..#..
  },
  { 0x48,  //.#..#... 0x9C
    0x7E,  //.######.
    0x49,  //.#..#..#
    0x43,  //.#....##
    0x66   //.##..##.
  },
  { 0x2B,  //..#.#.## 0x9D
    0x2F,  //..#.####
    0xFC,  //######..
    0x2F,  //..#.####
    0x2B   //..#.#.##
  },
  { 0xFF,  //######## 0x9E
    0x09,  //....#..#
    0x29,  //..#.#..#
    0xF6,  //####.##.
    0x20   //..#.....
  },
  { 0xC0,  //##...... 0x9F
    0x88,  //#...#...
    0x7E,  //.######.
    0x09,  //....#..#
    0x03   //......##
  },
  { 0x20,  //..#..... 0xA0
    0x54,  //.#.#.#..
    0x54,  //.#.#.#..
    0x79,  //.####..#
    0x41   //.#.....#
  },
  { 0x00,  //........ 0xA1
    0x00,  //........
    0x44,  //.#...#..
    0x7D,  //.#####.#
    0x41   //.#.....#
  },
  { 0x30,  //..##.... 0xA2
    0x48,  //.#..#...
    0x48,  //.#..#...
    0x4A,  //.#..#.#.
    0x32   //..##..#.
  },
  { 0x38,  //..###... 0xA3
    0x40,  //.#......
    0x40,  //.#......
    0x22,  //..#...#.
    0x7A   //.####.#.
  },
  { 0x00,  //........ 0xA4
    0x7A,  //.####.#.
    0x0A,  //....#.#.
    0x0A,  //....#.#.
    0x72   //.###..#.
  },
  { 0x7D,  //.#####.# 0xA5
    0x0D,  //....##.#
    0x19,  //...##..#
    0x31,  //..##...#
    0x7D   //.#####.#
  },
  { 0x26,  //..#..##. 0xA6
    0x29,  //..#.#..#
    0x29,  //..#.#..#
    0x2F,  //..#.####
    0x28   //..#.#...
  },
  { 0x26,  //..#..##. 0xA7
    0x29,  //..#.#..#
    0x29,  //..#.#..#
    0x29,  //..#.#..#
    0x26   //..#..##.
  },
  { 0x30,  //..##.... 0xA8
    0x48,  //.#..#...
    0x4D,  //.#..##.#
    0x40,  //.#......
    0x20   //..#.....
  },
  { 0x38,  //..###... 0xA9
    0x08,  //....#...
    0x08,  //....#...
    0x08,  //....#...
    0x08   //....#...
  },
  { 0x08,  //....#... 0xAA
    0x08,  //....#...
    0x08,  //....#...
    0x08,  //....#...
    0x38   //..###...
  },
  { 0x2F,  //..#.#### 0xAB
    0x10,  //...#....
    0xC8,  //##..#...
    0xAC,  //#.#.##..
    0xBA   //#.###.#.
  },
  { 0x2F,  //..#.#### 0xAC
    0x10,  //...#....
    0x28,  //..#.#...
    0x34,  //..##.#..
    0xFA   //#####.#.
  },
  { 0x00,  //........ 0xAD
    0x00,  //........
    0x7B,  //.####.##
    0x00,  //........
    0x00   //........
  },
  { 0x08,  //....#... 0xAE
    0x14,  //...#.#..
    0x2A,  //..#.#.#.
    0x14,  //...#.#..
    0x22   //..#...#.
  },
  { 0x22,  //..#...#. 0xAF
    0x14,  //...#.#..
    0x2A,  //..#.#.#.
    0x14,  //...#.#..
    0x08   //....#...
  },
  { 0x55,  //.#.#.#.# 0xB0
    0x00,  //........
    0x55,  //.#.#.#.#
    0x00,  //........
    0x55   //.#.#.#.#
  },
  { 0xAA,  //#.#.#.#. 0xB1
    0x55,  //.#.#.#.#
    0xAA,  //#.#.#.#.
    0x55,  //.#.#.#.#
    0xAA   //#.#.#.#.
  },
  { 0xFF,  //######## 0xB2
    0x55,  //.#.#.#.#
    0xFF,  //########
    0x55,  //.#.#.#.#
    0xFF   //########
  },
  { 0x00,  //........ 0xB3
    0x00,  //........
    0x00,  //........
    0xFF,  //########
    0x00   //........
  },
  { 0x10,  //...#.... 0xB4
    0x10,  //...#....
    0x10,  //...#....
    0xFF,  //########
    0x00   //........
  },
  { 0x14,  //...#.#.. 0xB5
    0x14,  //...#.#..
    0x14,  //...#.#..
    0xFF,  //########
    0x00   //........
  },
  { 0x10,  //...#.... 0xB6
    0x10,  //...#....
    0xFF,  //########
    0x00,  //........
    0xFF   //########
  },
  { 0x10,  //...#.... 0xB7
    0x10,  //...#....
    0xF0,  //####....
    0x10,  //...#....
    0xF0   //####....
  },
  { 0x14,  //...#.#.. 0xB8
    0x14,  //...#.#..
    0x14,  //...#.#..
    0xFC,  //######..
    0x00   //........
  },
  { 0x14,  //...#.#.. 0xB9
    0x14,  //...#.#..
    0xF7,  //####.###
    0x00,  //........
    0xFF   //########
  },
  { 0x00,  //........ 0xBA
    0x00,  //........
    0xFF,  //########
    0x00,  //........
    0xFF   //########
  },
  { 0x14,  //...#.#.. 0xBB
    0x14,  //...#.#..
    0xF4,  //####.#..
    0x04,  //.....#..
    0xFC   //######..
  },
  { 0x14,  //...#.#.. 0xBC
    0x14,  //...#.#..
    0x17,  //...#.###
    0x10,  //...#....
    0x1F   //...#####
  },
  { 0x10,  //...#.... 0xBD
    0x10,  //...#....
    0x1F,  //...#####
    0x10,  //...#....
    0x1F   //...#####
  },
  { 0x14,  //...#.#.. 0xBE
    0x14,  //...#.#..
    0x14,  //...#.#..
    0x1F,  //...#####
    0x00   //........
  },
  { 0x10,  //...#.... 0xBF
    0x10,  //...#....
    0x10,  //...#....
    0xF0,  //####....
    0x00   //........
  },
  { 0x00,  //........ 0xC0
    0x00,  //........
    0x00,  //........
    0x1F,  //...#####
    0x10   //...#....
  },
  { 0x10,  //...#.... 0xC1
    0x10,  //...#....
    0x10,  //...#....
    0x1F,  //...#####
    0x10   //...#....
  },
  { 0x10,  //...#.... 0xC2
    0x10,  //...#....
    0x10,  //...#....
    0xF0,  //####....
    0x10   //...#....
  },
  { 0x00,  //........ 0xC3
    0x00,  //........
    0x00,  //........
    0xFF,  //########
    0x10   //...#....
  },
  { 0x10,  //...#.... 0xC4
    0x10,  //...#....
    0x10,  //...#....
    0x10,  //...#....
    0x10   //...#....
  },
  { 0x10,  //...#.... 0xC5
    0x10,  //...#....
    0x10,  //...#....
    0xFF,  //########
    0x10   //...#....
  },
  { 0x00,  //........ 0xC6
    0x00,  //........
    0x00,  //........
    0xFF,  //########
    0x14   //...#.#..
  },
  { 0x00,  //........ 0xC7
    0x00,  //........
    0xFF,  //########
    0x00,  //........
    0xFF   //########
  },
  { 0x00,  //........ 0xC8
    0x00,  //........
    0x1F,  //...#####
    0x10,  //...#....
    0x17   //...#.###
  },
  { 0x00,  //........ 0xC9
    0x00,  //........
    0xFC,  //######..
    0x04,  //.....#..
    0xF4   //####.#..
  },
  { 0x14,  //...#.#.. 0xCA
    0x14,  //...#.#..
    0x17,  //...#.###
    0x10,  //...#....
    0x17   //...#.###
  },
  { 0x14,  //...#.#.. 0xCB
    0x14,  //...#.#..
    0xF4,  //####.#..
    0x04,  //.....#..
    0xF4   //####.#..
  },
  { 0x00,  //........ 0xCC
    0x00,  //........
    0xFF,  //########
    0x00,  //........
    0xF7   //####.###
  },
  { 0x14,  //...#.#.. 0xCD
    0x14,  //...#.#..
    0x14,  //...#.#..
    0x14,  //...#.#..
    0x14   //...#.#..
  },
  { 0x14,  //...#.#.. 0xCE
    0x14,  //...#.#..
    0xF7,  //####.###
    0x00,  //........
    0xF7   //####.###
  },
  { 0x14,  //...#.#.. 0xCF
    0x14,  //...#.#..
    0x14,  //...#.#..
    0x17,  //...#.###
    0x14   //...#.#..
  },
  { 0x10,  //...#.... 0xD0
    0x10,  //...#....
    0x1F,  //...#####
    0x10,  //...#....
    0x1F   //...#####
  },
  { 0x14,  //...#.#.. 0xD1
    0x14,  //...#.#..
    0x14,  //...#.#..
    0xF4,  //####.#..
    0x14   //...#.#..
  },
  { 0x10,  //...#.... 0xD2
    0x10,  //...#....
    0xF0,  //####....
    0x10,  //...#....
    0xF0   //####....
  },
  { 0x00,  //........ 0xD3
    0x00,  //........
    0x1F,  //...#####
    0x10,  //...#....
    0x1F   //...#####
  },
  { 0x00,  //........ 0xD4
    0x00,  //........
    0x00,  //........
    0x1F,  //...#####
    0x14   //...#.#..
  },
  { 0x00,  //........ 0xD5
    0x00,  //........
    0x00,  //........
    0xFC,  //######..
    0x14   //...#.#..
  },
  { 0x00,  //........ 0xD6
    0x00,  //........
    0xF0,  //####....
    0x10,  //...#....
    0xF0   //####....
  },
  { 0x10,  //...#.... 0xD7
    0x10,  //...#....
    0xFF,  //########
    0x10,  //...#....
    0xFF   //########
  },
  { 0x14,  //...#.#.. 0xD8
    0x14,  //...#.#..
    0x14,  //...#.#..
    0xFF,  //########
    0x14   //...#.#..
  },
  { 0x10,  //...#.... 0xD9
    0x10,  //...#....
    0x10,  //...#....
    0x1F,  //...#####
    0x00   //........
  },
  { 0x00,  //........ 0xDA
    0x00,  //........
    0x00,  //........
    0xF0,  //####....
    0x10   //...#....
  },
  { 0xFF,  //######## 0xDB
    0xFF,  //########
    0xFF,  //########
    0xFF,  //########
    0xFF   //########
  },
  { 0xF0,  //####.... 0xDC
    0xF0,  //####....
    0xF0,  //####....
    0xF0,  //####....
    0xF0   //####....
  },
  { 0xFF,  //######## 0xDD
    0xFF,  //########
    0xFF,  //########
    0x00,  //........
    0x00   //........
  },
  { 0x00,  //........ 0xDE
    0x00,  //........
    0x00,  //........
    0xFF,  //########
    0xFF   //########
  },
  { 0x0F,  //....#### 0xDF
    0x0F,  //....####
    0x0F,  //....####
    0x0F,  //....####
    0x0F   //....####
  },
  { 0x38,  //..###... 0xE0
    0x44,  //.#...#..
    0x44,  //.#...#..
    0x38,  //..###...
    0x44   //.#...#..
  },
  { 0xFC,  //######.. 0xE1
    0x4A,  //.#..#.#.
    0x4A,  //.#..#.#.
    0x4A,  //.#..#.#.
    0x34   //..##.#..
  },
  { 0x7E,  //.######. 0xE2
    0x02,  //......#.
    0x02,  //......#.
    0x06,  //.....##.
    0x06   //.....##.
  },
  { 0x02,  //......#. 0xE3
    0x7E,  //.######.
    0x02,  //......#.
    0x7E,  //.######.
    0x02   //......#.
  },
  { 0x63,  //.##...## 0xE4
    0x55,  //.#.#.#.#
    0x49,  //.#..#..#
    0x41,  //.#.....#
    0x63   //.##...##
  },
  { 0x38,  //..###... 0xE5
    0x44,  //.#...#..
    0x44,  //.#...#..
    0x3C,  //..####..
    0x04   //.....#..
  },
  { 0x40,  //.#...... 0xE6
    0x7E,  //.######.
    0x20,  //..#.....
    0x1E,  //...####.
    0x20   //..#.....
  },
  { 0x06,  //.....##. 0xE7
    0x02,  //......#.
    0x7E,  //.######.
    0x02,  //......#.
    0x02   //......#.
  },
  { 0x99,  //#..##..# 0xE8
    0xA5,  //#.#..#.#
    0xE7,  //###..###
    0xA5,  //#.#..#.#
    0x99   //#..##..#
  },
  { 0x1C,  //...###.. 0xE9
    0x2A,  //..#.#.#.
    0x49,  //.#..#..#
    0x2A,  //..#.#.#.
    0x1C   //...###..
  },
  { 0x4C,  //.#..##.. 0xEA
    0x72,  //.###..#.
    0x01,  //.......#
    0x72,  //.###..#.
    0x4C   //.#..##..
  },
  { 0x30,  //..##.... 0xEB
    0x4A,  //.#..#.#.
    0x4D,  //.#..##.#
    0x4D,  //.#..##.#
    0x30   //..##....
  },
  { 0x30,  //..##.... 0xEC
    0x48,  //.#..#...
    0x78,  //.####...
    0x48,  //.#..#...
    0x30   //..##....
  },
  { 0xBC,  //#.####.. 0xED
    0x62,  //.##...#.
    0x5A,  //.#.##.#.
    0x46,  //.#...##.
    0x3D   //..####.#
  },
  { 0x3E,  //..#####. 0xEE
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x49,  //.#..#..#
    0x00   //........
  },
  { 0x7E,  //.######. 0xEF
    0x01,  //.......#
    0x01,  //.......#
    0x01,  //.......#
    0x7E   //.######.
  },
  { 0x2A,  //..#.#.#. 0xF0
    0x2A,  //..#.#.#.
    0x2A,  //..#.#.#.
    0x2A,  //..#.#.#.
    0x2A   //..#.#.#.
  },
  { 0x44,  //.#...#.. 0xF1
    0x44,  //.#...#..
    0x5F,  //.#.#####
    0x44,  //.#...#..
    0x44   //.#...#..
  },
  { 0x40,  //.#...... 0xF2
    0x51,  //.#.#...#
    0x4A,  //.#..#.#.
    0x44,  //.#...#..
    0x40   //.#......
  },
  { 0x40,  //.#...... 0xF3
    0x44,  //.#...#..
    0x4A,  //.#..#.#.
    0x51,  //.#.#...#
    0x40   //.#......
  },
  { 0x00,  //........ 0xF4
    0x00,  //........
    0xFF,  //########
    0x01,  //.......#
    0x03   //......##
  },
  { 0xE0,  //###..... 0xF5
    0x80,  //#.......
    0xFF,  //########
    0x00,  //........
    0x00   //........
  },
  { 0x08,  //....#... 0xF6
    0x08,  //....#...
    0x6B,  //.##.#.##
    0x6B,  //.##.#.##
    0x08   //....#...
  },
  { 0x36,  //..##.##. 0xF7
    0x12,  //...#..#.
    0x36,  //..##.##.
    0x24,  //..#..#..
    0x36   //..##.##.
  },
  { 0x06,  //.....##. 0xF8
    0x0F,  //....####
    0x09,  //....#..#
    0x0F,  //....####
    0x06   //.....##.
  },
  { 0x00,  //........ 0xF9
    0x00,  //........
    0x18,  //...##...
    0x18,  //...##...
    0x00   //........
  },
  { 0x00,  //........ 0xFA
    0x00,  //........
    0x10,  //...#....
    0x10,  //...#....
    0x00   //........
  },
  { 0x30,  //..##.... 0xFB
    0x40,  //.#......
    0xFF,  //########
    0x01,  //.......#
    0x01   //.......#
  },
  { 0x00,  //........ 0xFC
    0x1F,  //...#####
    0x01,  //.......#
    0x01,  //.......#
    0x1E   //...####.
  },
  { 0x00,  //........ 0xFD
    0x19,  //...##..#
    0x1D,  //...###.#
    0x17,  //...#.###
    0x12   //...#..#.
  },
  { 0x00,  //........ 0xFE
    0x3C,  //..####..
    0x3C,  //..####..
    0x3C,  //..####..
    0x3C   //..####..
  },
  { 0x00,  //........ 0xFF
    0x00,  //........
    0x00,  //........
    0x00,  //........
    0x00   //........
  }
};
